import copy

import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 63:
    pytest.skip("ICU4C<63", allow_module_level=True)

from icupy import (
    ErrorCode, Formattable, Locale, MeasureUnit, UErrorCode,
    UNumberRangeCollapse, UNumberRangeIdentityFallback,
    UNumberRangeIdentityResult, UnicodeString, UnicodeStringAppendable,
)
from icupy.number import (
    FormattedNumberRange, LocalizedNumberRangeFormatter, NumberFormatter,
    NumberRangeFormatter, UnlocalizedNumberRangeFormatter,
)


def test_formatted_number_range_63():
    fmt = (NumberRangeFormatter.with_locale("en-US")
           .number_formatter_first(NumberFormatter.with_()
                                   .unit(MeasureUnit.create_meter()))
           .number_formatter_second(NumberFormatter.with_()
                                    .unit(MeasureUnit.create_kilometer())))
    assert isinstance(fmt, LocalizedNumberRangeFormatter)

    num = fmt.format_formattable_range(Formattable(3), Formattable(3000))
    assert isinstance(num, FormattedNumberRange)

    # Appendable &icu::number::FormattedNumberRange::appendTo(
    #       Appendable &appendable,
    #       UErrorCode &status
    # )
    s = UnicodeString()
    appendable = UnicodeStringAppendable(s)
    result = num.append_to(appendable)
    assert isinstance(result, UnicodeStringAppendable)
    assert id(result) == id(appendable)
    assert s == "3 m \u2013 3,000 km"  # 3 m – 3,000 km

    # UNumberRangeIdentityResult
    # icu::number::FormattedNumberRange::getIdentityResult(UErrorCode &status)
    assert (num.get_identity_result()
            == UNumberRangeIdentityResult.UNUM_IDENTITY_RESULT_NOT_EQUAL)

    # UnicodeString icu::number::FormattedNumberRange::toString(
    #       UErrorCode &status
    # )
    result = num.to_string()
    assert isinstance(result, UnicodeString)
    assert result == "3 m \u2013 3,000 km"  # 3 m – 3,000 km


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_formatted_number_range_64():
    from icupy import FormattedValue

    assert issubclass(FormattedNumberRange, FormattedValue)

    fmt = (NumberRangeFormatter.with_locale("en-US")
           .number_formatter_first(NumberFormatter.with_()
                                   .unit(MeasureUnit.create_meter()))
           .number_formatter_second(NumberFormatter.with_()
                                    .unit(MeasureUnit.create_kilometer())))
    assert isinstance(fmt, LocalizedNumberRangeFormatter)

    num = fmt.format_formattable_range(Formattable(3), Formattable(3000))
    assert isinstance(num, FormattedNumberRange)

    # UnicodeString icu::number::FormattedNumberRange::toTempString(
    #       UErrorCode &status
    # )
    result = num.to_temp_string()
    assert isinstance(result, UnicodeString)
    assert result == "3 m \u2013 3,000 km"  # 3 m – 3,000 km


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_formatted_number_range_68():
    fmt = (NumberRangeFormatter.with_locale("en-US")
           .number_formatter_first(NumberFormatter.with_()
                                   .unit(MeasureUnit.create_meter()))
           .number_formatter_second(NumberFormatter.with_()
                                    .unit(MeasureUnit.create_kilometer())))
    assert isinstance(fmt, LocalizedNumberRangeFormatter)

    num = fmt.format_formattable_range(Formattable(3), Formattable(3000))
    assert isinstance(num, FormattedNumberRange)

    # template<typename StringClass>
    # std::pair<StringClass, StringClass>
    # icu::number::FormattedNumberRange::getDecimalNumbers(UErrorCode &status)
    pair = num.get_decimal_numbers()
    assert isinstance(pair, tuple)
    assert len(pair) == 2
    assert all(isinstance(x, str) for x in pair)
    assert pair[0] == "3"
    assert pair[1] == "3E+3"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_formatted_number_range_69():
    from icupy import (
        ConstrainedFieldPosition, UFieldCategory, UNumberFormatFields,
    )

    fmt = (NumberRangeFormatter.with_locale("en-US")
           .number_formatter_first(NumberFormatter.with_()
                                   .unit(MeasureUnit.create_meter()))
           .number_formatter_second(NumberFormatter.with_()
                                    .unit(MeasureUnit.create_kilometer())))
    assert isinstance(fmt, LocalizedNumberRangeFormatter)

    # "3 m – 3,000 km"
    num = fmt.format_formattable_range(Formattable(3), Formattable(3000))
    assert isinstance(num, FormattedNumberRange)

    # UBool icu::number::FormattedNumberRange::nextPosition(
    #       ConstrainedFieldPosition &cfpos,
    #       UErrorCode &status
    # )
    cfpos = ConstrainedFieldPosition()
    assert num.next_position(cfpos)
    assert (cfpos.get_category()
            == UFieldCategory.UFIELD_CATEGORY_NUMBER_RANGE_SPAN)
    assert cfpos.get_field() == UNumberFormatFields.UNUM_INTEGER_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 3)  # "3 m"

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_INTEGER_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 1)  # "3"

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_MEASURE_UNIT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (2, 3)  # "m"

    assert num.next_position(cfpos)
    assert (cfpos.get_category()
            == UFieldCategory.UFIELD_CATEGORY_NUMBER_RANGE_SPAN)
    assert cfpos.get_field() == UNumberFormatFields.UNUM_FRACTION_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (6, 14)  # "3,000 km"

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert (cfpos.get_field()
            == UNumberFormatFields.UNUM_GROUPING_SEPARATOR_FIELD)
    assert (cfpos.get_start(), cfpos.get_limit()) == (7, 8)  # ","

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_INTEGER_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (6, 11)  # "3,000"

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_MEASURE_UNIT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (12, 14)  # "km"

    assert not num.next_position(cfpos)


def test_localized_number_range_formatter_63():
    # [1]
    # icu::number::LocalizedNumberRangeFormatter::LocalizedNumberRangeFormatter()
    fmt = LocalizedNumberRangeFormatter()

    # [2]
    # icu::number::LocalizedNumberRangeFormatter::LocalizedNumberRangeFormatter(
    #       const LocalizedNumberRangeFormatter &other
    # )
    fmt2 = LocalizedNumberRangeFormatter(fmt)

    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::collapse(
    #       UNumberRangeCollapse collapse
    # )
    result = fmt2.collapse(UNumberRangeCollapse.UNUM_RANGE_COLLAPSE_UNIT)
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # template<typename Derived>
    # UBool icu::number::NumberRangeFormatterSettings<Derived>::copyErrorTo(
    #       UErrorCode &outErrorCode
    # )
    out_error_code = ErrorCode()
    assert not fmt2.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR

    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::identityFallback(
    #       UNumberRangeIdentityFallback identityFallback
    # )
    result = fmt2.identity_fallback(
        UNumberRangeIdentityFallback.UNUM_IDENTITY_FALLBACK_RANGE)
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterBoth(
    #       const UnlocalizedNumberFormatter &formatter
    # )
    result = fmt2.number_formatter_both(NumberFormatter.with_())
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterFirst(
    #       const UnlocalizedNumberFormatter &formatterFirst
    # )
    result = fmt2.number_formatter_first(NumberFormatter.with_())
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterSecond(
    #       const UnlocalizedNumberFormatter &formatterSecond
    # )
    result = fmt2.number_formatter_second(NumberFormatter.with_())
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # FormattedNumberRange
    # icu::number::LocalizedNumberRangeFormatter::formatFormattableRange(
    #       const Formattable &first,
    #       const Formattable &second,
    #       UErrorCode &status
    # )
    formatter = NumberFormatter.with_().unit(MeasureUnit.create_kilometer())
    result = (fmt2.number_formatter_both(formatter)
              .format_formattable_range(Formattable(4), Formattable(6)))
    assert isinstance(result, FormattedNumberRange)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_localized_number_range_formatter_64():
    formatter = NumberFormatter.with_().unit(MeasureUnit.create_meter())
    fmt = (NumberRangeFormatter.with_locale("en-US")
           .collapse(UNumberRangeCollapse.UNUM_RANGE_COLLAPSE_NONE)
           .number_formatter_both(formatter))
    assert isinstance(fmt, LocalizedNumberRangeFormatter)
    assert (fmt.format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    # template<typename Derived>
    # LocalPointer<Derived>
    # icu::number::NumberRangeFormatterSettings<Derived>::clone()
    fmt2 = fmt.clone()
    assert isinstance(fmt2, LocalizedNumberRangeFormatter)
    assert (fmt2.format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    fmt3 = copy.copy(fmt)
    assert (fmt3.format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    fmt4 = copy.deepcopy(fmt)
    assert (fmt4.format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m


def test_number_range_formatter():
    # static UnlocalizedNumberRangeFormatter
    # icu::number::NumberRangeFormatter::with()
    fmt = NumberRangeFormatter.with_()
    assert isinstance(fmt, UnlocalizedNumberRangeFormatter)

    # static LocalizedNumberRangeFormatter
    # icu::number::NumberRangeFormatter::withLocale(const Locale &locale)
    fmt = NumberRangeFormatter.with_locale(Locale.get_us())
    assert isinstance(fmt, LocalizedNumberRangeFormatter)

    fmt = NumberRangeFormatter.with_locale("en-US")
    assert isinstance(fmt, LocalizedNumberRangeFormatter)


def test_unlocalized_number_range_formatter_63():
    # [1]
    # icu::number::UnlocalizedNumberRangeFormatter::UnlocalizedNumberRangeFormatter()
    fmt = UnlocalizedNumberRangeFormatter()

    # [2]
    # icu::number::UnlocalizedNumberRangeFormatter::UnlocalizedNumberRangeFormatter(
    #       const UnlocalizedNumberRangeFormatter &other
    # )
    fmt2 = UnlocalizedNumberRangeFormatter(fmt)

    # LocalizedNumberRangeFormatter
    # icu::number::UnlocalizedNumberRangeFormatter::locale(
    #       const Locale &locale
    # )
    result = fmt2.locale(Locale.get_us())
    assert isinstance(result, LocalizedNumberRangeFormatter)

    result = fmt2.locale("en-US")
    assert isinstance(result, LocalizedNumberRangeFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::collapse(
    #       UNumberRangeCollapse collapse
    # )
    result = fmt2.collapse(UNumberRangeCollapse.UNUM_RANGE_COLLAPSE_UNIT)
    assert isinstance(result, UnlocalizedNumberRangeFormatter)

    # template<typename Derived>
    # UBool icu::number::NumberRangeFormatterSettings<Derived>::copyErrorTo(
    #       UErrorCode &outErrorCode
    # )
    out_error_code = ErrorCode()
    assert not fmt2.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR

    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::identityFallback(
    #       UNumberRangeIdentityFallback identityFallback
    # )
    result = fmt2.identity_fallback(
        UNumberRangeIdentityFallback.UNUM_IDENTITY_FALLBACK_RANGE)
    assert isinstance(result, UnlocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterBoth(
    #       const UnlocalizedNumberFormatter &formatter
    # )
    result = fmt2.number_formatter_both(NumberFormatter.with_())
    assert isinstance(result, UnlocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterFirst(
    #       const UnlocalizedNumberFormatter &formatterFirst
    # )
    result = fmt2.number_formatter_first(NumberFormatter.with_())
    assert isinstance(result, UnlocalizedNumberRangeFormatter)

    # [2]
    # template<typename Derived>
    # Derived icu::number::NumberRangeFormatterSettings<Derived>::numberFormatterSecond(
    #       const UnlocalizedNumberFormatter &formatterSecond
    # )
    result = fmt2.number_formatter_second(NumberFormatter.with_())
    assert isinstance(result, UnlocalizedNumberRangeFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_unlocalized_number_range_formatter_64():
    formatter = NumberFormatter.with_().unit(MeasureUnit.create_meter())
    fmt = (NumberRangeFormatter.with_()
           .collapse(UNumberRangeCollapse.UNUM_RANGE_COLLAPSE_NONE)
           .number_formatter_both(formatter))
    assert isinstance(fmt, UnlocalizedNumberRangeFormatter)
    assert (fmt.locale("en-US")
            .format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    # template<typename Derived>
    # LocalPointer<Derived>
    # icu::number::NumberRangeFormatterSettings<Derived>::clone()
    fmt2 = fmt.clone()
    assert isinstance(fmt2, UnlocalizedNumberRangeFormatter)
    assert (fmt2.locale("en-US")
            .format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    fmt3 = copy.copy(fmt)
    assert (fmt3.locale("en-US")
            .format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m

    fmt4 = copy.deepcopy(fmt)
    assert (fmt4.locale("en-US")
            .format_formattable_range(Formattable(1), Formattable(5))
            .to_string()) == "1 m \u2013 5 m"  # 1 m - 5 m



from icupy import (
    CharacterIterator, ForwardCharacterIterator, StringCharacterIterator,
    UCharCharacterIterator, UnicodeString,
)


def test_iter():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # int32_t icu::CharacterIterator::getLength(void)
    assert it.get_length() == len(it) == 4

    assert list(it) == ["a", "\U0001f338", "b"]

    t = [it.first()] + [
        c for c in iter(it.next, StringCharacterIterator.DONE)]
    assert t == [0x61, 0xd83c, 0xdf38, 0x62]

    t = [it.first32()] + [
        c for c in iter(it.next32, StringCharacterIterator.DONE)]
    assert t == [0x61, 0x1f338, 0x62]

    assert reversed(it) == ["b", "\U0001f338", "a"]

    src = UnicodeString("\\uD83C\\uDF38").unescape()
    it = StringCharacterIterator(src)
    assert list(it) == ["\U0001f338"]
    assert reversed(it) == ["\U0001f338"]

    src = UnicodeString()
    it = StringCharacterIterator(src)
    assert list(it) == []
    assert reversed(it) == []


def test_move():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # int32_t icu::CharacterIterator::move(
    #       int32_t delta,
    #       EOrigin origin
    # )
    assert it.move(1, StringCharacterIterator.START) == 1
    assert it.move(1, StringCharacterIterator.CURRENT) == 2
    assert it.move(1, StringCharacterIterator.END) == 4


def test_move32():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # int32_t icu::CharacterIterator::move32(
    #       int32_t delta,
    #       EOrigin origin
    # )
    assert it.move32(1, StringCharacterIterator.START) == 1
    assert it.move32(1, StringCharacterIterator.CURRENT) == 3
    assert it.move32(1, StringCharacterIterator.END) == 4


def test_next():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    t = list()

    # int32_t icu::CharacterIterator::setToStart()
    assert it.set_to_start() == 0

    # UBool icu::ForwardCharacterIterator::hasNext()
    while it.has_next():
        # char16_t icu::CharacterIterator::current(void)
        current = it.current()

        # char16_t icu::ForwardCharacterIterator::nextPostInc(void)
        c = it.next_post_inc()
        assert c == current
        t.append(c)
    assert t == [0x61, 0xd83c, 0xdf38, 0x62]

    # char16_t icu::CharacterIterator::firstPostInc(void)
    c = it.first_post_inc()
    t.clear()
    while c != StringCharacterIterator.DONE:
        t.append(c)
        c = it.next_post_inc()
    assert t == [0x61, 0xd83c, 0xdf38, 0x62]

    # char16_t icu::CharacterIterator::first(void)
    c = it.first()
    t.clear()
    while c != StringCharacterIterator.DONE:
        current = it.current()
        assert c == current
        t.append(c)

        # char16_t icu::CharacterIterator::next(void)
        c = it.next()
    assert t == [0x61, 0xd83c, 0xdf38, 0x62]


def test_next32():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    t = list()
    it.set_to_start()
    while it.has_next():
        # UChar32 icu::CharacterIterator::current32(void)
        current = it.current32()

        # UChar32 icu::ForwardCharacterIterator::next32PostInc(void)
        c = it.next32_post_inc()
        assert c == current
        t.append(c)
    assert t == [0x61, 0x1f338, 0x62]

    # UChar32 icu::CharacterIterator::first32PostInc(void)
    c = it.first32_post_inc()
    t.clear()
    while c != StringCharacterIterator.DONE:
        t.append(c)
        c = it.next32_post_inc()
    assert t == [0x61, 0x1f338, 0x62]

    # UChar32 icu::CharacterIterator::first32(void)
    c = it.first32()
    t.clear()
    while c != StringCharacterIterator.DONE:
        current = it.current32()
        assert c == current
        t.append(c)

        # UChar32 icu::CharacterIterator::next32(void)
        c = it.next32()
    assert t == [0x61, 0x1f338, 0x62]


def test_previous():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    t = list()

    # int32_t icu::CharacterIterator::setToEnd()
    assert it.set_to_end() == 4

    # UBool icu::CharacterIterator::hasPrevious()
    while it.has_previous():
        # char16_t icu::CharacterIterator::previous(void)
        c = it.previous()
        current = it.current()
        assert c == current
        t.append(c)
    assert t == [0x62, 0xdf38, 0xd83c, 0x61]

    # char16_t icu::CharacterIterator::last(void)
    c = it.last()
    t.clear()
    while c != StringCharacterIterator.DONE:
        t.append(c)
        c = it.previous()
    assert t == [0x62, 0xdf38, 0xd83c, 0x61]


def test_previous32():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    t = list()
    it.set_to_end()
    while it.has_previous():
        # UChar32 icu::CharacterIterator::previous32(void)
        c = it.previous32()
        current = it.current32()
        assert c == current
        t.append(c)
    assert t == [0x62, 0x1f338, 0x61]

    # UChar32 icu::CharacterIterator::last32(void)
    c = it.last32()
    t.clear()
    while c != StringCharacterIterator.DONE:
        t.append(c)
        c = it.previous32()
    assert t == [0x62, 0x1f338, 0x61]


def test_set_index():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # int32_t icu::CharacterIterator::getIndex(void)
    assert it.get_index() == 0

    # char16_t icu::CharacterIterator::setIndex(int32_t position)
    assert it.set_index(0) == 0x61
    assert it.get_index() == 0

    assert it.set_index(1) == 0xd83c
    assert it.get_index() == 1

    assert it.set_index(2) == 0xdf38
    assert it.get_index() == 2

    assert it.set_index(3) == 0x62
    assert it.get_index() == 3

    # int32_t icu::CharacterIterator::endIndex(void)
    assert it.end_index() == 4

    # int32_t icu::CharacterIterator::startIndex(void)
    assert it.start_index() == 0


def test_set_index32():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # UChar32 icu::CharacterIterator::setIndex32(int32_t position)
    assert it.set_index32(0) == 0x61
    assert it.get_index() == 0

    assert it.set_index32(1) == 0x1f338
    assert it.get_index() == 1

    assert it.set_index32(2) == 0x1f338
    assert it.get_index() == 1

    assert it.set_index32(3) == 0x62
    assert it.get_index() == 3


def test_set_text():
    src = UnicodeString("a\\uD83C\\uDF38b").unescape()
    it = StringCharacterIterator(src)

    # [2]
    # void icu::StringCharacterIterator::setText(const UnicodeString &newText)
    it.set_text(UnicodeString("foo bar baz"))

    # void icu::CharacterIterator::getText(UnicodeString &result)
    result = UnicodeString()
    it.get_text(result)
    assert result == "foo bar baz"

    new_text = "a\U0001f338b"
    it.set_text(new_text)
    it.get_text(result)
    assert result == new_text


def test_string_character_iterator():
    assert issubclass(CharacterIterator, ForwardCharacterIterator)
    assert issubclass(UCharCharacterIterator, CharacterIterator)
    assert issubclass(StringCharacterIterator, UCharCharacterIterator)

    text_str = UnicodeString("a\\uD83C\\uDF38b").unescape()
    expected = "a\U0001f338b"
    dest = UnicodeString()

    # [1]
    # icu::StringCharacterIterator::StringCharacterIterator(
    #       const UnicodeString &textStr
    # )
    it1 = StringCharacterIterator(text_str)
    assert it1.get_length() == 4
    assert len(it1) == 4
    dest.remove()
    it1.get_text(dest)
    assert dest == expected

    it1a = StringCharacterIterator("a\U0001f338b")
    assert it1a.get_length() == 4
    assert len(it1a) == 4
    dest.remove()
    it1a.get_text(dest)
    assert dest == expected

    # [2]
    # icu::StringCharacterIterator::StringCharacterIterator(
    #       const UnicodeString &textStr,
    #       int32_t textPos
    # )
    it2 = StringCharacterIterator(text_str, 2)
    assert it2.get_length() == 4
    assert len(it2) == 4
    dest.remove()
    it2.get_text(dest)
    assert dest == expected
    assert it2.current() == 0xdf38

    it2a = StringCharacterIterator("a\U0001f338b", 2)
    assert it2a.get_length() == 4
    assert len(it2a) == 4
    dest.remove()
    it2a.get_text(dest)
    assert dest == expected
    assert it2a.current() == 0xdf38

    # [3]
    # icu::StringCharacterIterator::StringCharacterIterator(
    #       const UnicodeString &textStr,
    #       int32_t textBegin,
    #       int32_t textEnd,
    #       int32_t textPos
    # )
    it3 = StringCharacterIterator(text_str, 1, 3, 0)
    assert it3.get_length() == 4
    assert len(it3) == 4
    dest.remove()
    it3.get_text(dest)
    assert dest == expected
    assert it3.current() == 0xd83c
    assert it3.next() == 0xdf38
    assert it3.next() == StringCharacterIterator.DONE
    assert it3.first() == 0xd83c

    it3a = StringCharacterIterator("a\U0001f338b", 1, 3, 0)
    assert it3a.get_length() == 4
    assert len(it3a) == 4
    dest.remove()
    it3a.get_text(dest)
    assert dest == expected
    assert it3a.current() == 0xd83c
    assert it3a.next() == 0xdf38
    assert it3a.next() == StringCharacterIterator.DONE
    assert it3a.first() == 0xd83c

    # [4]
    # icu::StringCharacterIterator::StringCharacterIterator(
    #       const StringCharacterIterator &that
    # )
    it1.set_index(2)
    it4 = StringCharacterIterator(it1)
    assert it4.get_length() == 4
    assert len(it4) == 4
    dest.remove()
    it4.get_text(dest)
    assert dest == expected
    assert it4.get_index() == 2
    assert it4.current() == 0xdf38



from icupy import (
    MessagePattern, UMSGPAT_NO_NUMERIC_VALUE, UMessagePatternApostropheMode,
    UMessagePatternArgType, UMessagePatternPartType, UParseError,
    UnicodeString,
)


def test_api():
    pattern = "I don't '{know}' {gender,select,female{h''er}other{h'im}}."
    mp1 = MessagePattern(pattern, None)

    # UnicodeString icu::MessagePattern::autoQuoteApostropheDeep()
    result = mp1.auto_quote_apostrophe_deep()
    assert isinstance(result, UnicodeString)
    assert (result
            == "I don''t '{know}' {gender,select,female{h''er}other{h''im}}.")

    # int32_t icu::MessagePattern::countParts()
    assert mp1.count_parts() == 16

    # UMessagePatternApostropheMode icu::MessagePattern::getApostropheMode()
    assert (mp1.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL)

    # void icu::MessagePattern::clear()
    mp1.clear()
    assert mp1.count_parts() == 0

    # void icu::MessagePattern::clearPatternAndSetApostropheMode(
    #       UMessagePatternApostropheMode mode
    # )
    mp1 = MessagePattern(pattern, None)
    mp1.clear_pattern_and_set_apostrophe_mode(
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED)
    assert mp1.count_parts() == 0
    assert (mp1.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED)

    # int32_t icu::MessagePattern::getLimitPartIndex(int32_t start)
    mp1 = MessagePattern(pattern, None)
    assert mp1.get_limit_part_index(0) == 15

    # const Part &icu::MessagePattern::getPart(int32_t i)
    part = mp1.get_part(4)
    assert isinstance(part, MessagePattern.Part)

    # double icu::MessagePattern::getNumericValue(const Part &part)
    assert mp1.get_numeric_value(part) == UMSGPAT_NO_NUMERIC_VALUE

    # UMessagePatternPartType icu::MessagePattern::getPartType(int32_t i)
    assert (mp1.get_part_type(4)
            == UMessagePatternPartType.UMSGPAT_PART_TYPE_ARG_START)

    # int32_t icu::MessagePattern::getPatternIndex(int32_t partIndex)
    assert mp1.get_pattern_index(4) == 17

    # const UnicodeString &icu::MessagePattern::getPatternString()
    result = mp1.get_pattern_string()
    assert isinstance(result, UnicodeString)
    assert result == pattern

    # double icu::MessagePattern::getPluralOffset(int32_t pluralStart)
    pattern2 = ("{0,plural,offset:1 "
                "one{another meter}"
                "other{{0,number,00.#} meters}}")
    mp2 = MessagePattern(pattern2, None)
    assert (mp2.get_part_type(3)
            == UMessagePatternPartType.UMSGPAT_PART_TYPE_ARG_INT)
    assert mp2.get_plural_offset(3) == 1

    # UnicodeString icu::MessagePattern::getSubstring(const Part &part)
    part = mp2.get_part(3)
    result = mp2.get_substring(part)
    assert isinstance(result, UnicodeString)
    assert result == "1"

    # int32_t icu::MessagePattern::hashCode()
    assert mp1.hash_code()
    assert mp1.hash_code() != mp2.hash_code()

    # UBool icu::MessagePattern::hasNamedArguments()
    assert mp1.has_named_arguments()
    assert not mp2.has_named_arguments()

    # UBool icu::MessagePattern::hasNumberedArguments()
    assert not mp1.has_numbered_arguments()
    assert mp2.has_numbered_arguments()

    # UBool icu::MessagePattern::operator!=(const MessagePattern &other)
    mp3 = MessagePattern(pattern, None)
    assert mp1 != mp2
    assert not (mp1 != mp3)
    assert mp2 != mp3

    # UBool icu::MessagePattern::operator==(const MessagePattern &other)
    assert not (mp1 == mp2)
    assert mp1 == mp3
    assert not (mp2 == mp3)

    # UBool icu::MessagePattern::partSubstringMatches(
    #       const Part &part,
    #       const UnicodeString &s
    # )
    assert mp2.part_substring_matches(part, UnicodeString("1"))
    assert mp2.part_substring_matches(part, "1")

    # static int32_t icu::MessagePattern::validateArgumentName(
    #       const UnicodeString &name
    # )
    assert MessagePattern.validate_argument_name(UnicodeString("3")) == 3
    assert MessagePattern.validate_argument_name("3") == 3


def test_message_pattern():
    # [1]
    # icu::MessagePattern::MessagePattern(UErrorCode &errorCode)
    mp1 = MessagePattern()
    assert mp1.count_parts() == 0

    # [2]
    # icu::MessagePattern::MessagePattern(
    #       UMessagePatternApostropheMode mode,
    #       UErrorCode &errorCode
    # )
    mp2 = MessagePattern(
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL)
    assert mp1 == mp2
    assert mp2.count_parts() == 0

    mp2a = MessagePattern(
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED)
    assert mp1 != mp2a

    # [3]
    # icu::MessagePattern::MessagePattern(
    #       const UnicodeString &pattern,
    #       UParseError *parseError,
    #       UErrorCode &errorCode
    # )
    pattern = UnicodeString("I see {many}")
    parse_error = UParseError()
    mp3 = MessagePattern(pattern, parse_error)
    assert parse_error.offset == 0
    assert mp3.count_parts() == 5

    mp3a = MessagePattern(pattern, None)
    assert mp3 == mp3a

    parse_error = UParseError()
    mp3b = MessagePattern(str(pattern), parse_error)
    assert parse_error.offset == 0
    assert mp3 == mp3b

    mp3c = MessagePattern(str(pattern), None)
    assert mp3 == mp3c

    # [4]
    # icu::MessagePattern::MessagePattern(const MessagePattern &other)
    mp4 = MessagePattern(mp3)
    assert mp3 == mp4
    assert mp4.count_parts() == 5


def test_parse():
    pattern = UnicodeString(
        "At {1,time,::jmm} on {1,date,::dMMMM}, "
        "there was {2} on planet {0,number}.")

    # MessagePattern &icu::MessagePattern::parse(
    #       const UnicodeString &pattern,
    #       UParseError *parseError,
    #       UErrorCode &errorCode
    # )
    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse(pattern, parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 19

    mp = MessagePattern()
    result = mp.parse(pattern, None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 19

    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse(str(pattern), parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 19

    mp = MessagePattern()
    result = mp.parse(str(pattern), None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 19


def test_parse_choice_style():
    pattern = UnicodeString(
        "0#are no files|1#is one file|1<are {0, number} files")

    # MessagePattern &icu::MessagePattern::parseChoiceStyle(
    #       const UnicodeString &pattern,
    #       UParseError *parseError,
    #       UErrorCode &errorCode
    # )
    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_choice_style(pattern, parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    result = mp.parse_choice_style(pattern, None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_choice_style(str(pattern), parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    result = mp.parse_choice_style(str(pattern), None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16


def test_parse_plural_style():
    pattern = UnicodeString(
        "one{#st file}two{#nd file}few{#rd file}other{#th file}")

    # MessagePattern &icu::MessagePattern::parsePluralStyle(
    #       const UnicodeString &pattern,
    #       UParseError *parseError,
    #       UErrorCode &errorCode
    # )
    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_plural_style(pattern, parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    result = mp.parse_plural_style(pattern, None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_plural_style(str(pattern), parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16

    mp = MessagePattern()
    result = mp.parse_plural_style(str(pattern), None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 16


def test_parse_select_style():
    pattern = UnicodeString("female{h''er}other{h'im}")

    # MessagePattern &icu::MessagePattern::parseSelectStyle(
    #       const UnicodeString &pattern,
    #       UParseError *parseError,
    #       UErrorCode &errorCode
    # )
    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_select_style(pattern, parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 8

    mp = MessagePattern()
    result = mp.parse_select_style(pattern, None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 8

    mp = MessagePattern()
    parse_error = UParseError()
    result = mp.parse_select_style(str(pattern), parse_error)
    assert parse_error.offset == 0
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 8

    mp = MessagePattern()
    result = mp.parse_select_style(str(pattern), None)
    assert isinstance(result, MessagePattern)
    assert id(result) == id(mp)
    assert mp.count_parts() == 8


def test_part():
    pattern = "I don't '{know}' {gender,select,female{h''er}other{h'im}}."
    mp3 = MessagePattern(pattern, None)

    part = mp3.get_part(4)
    assert isinstance(part, MessagePattern.Part)

    # UMessagePatternArgType icu::MessagePattern::Part::getArgType()
    assert (part.get_arg_type()
            == UMessagePatternArgType.UMSGPAT_ARG_TYPE_SELECT)

    # int32_t icu::MessagePattern::Part::getIndex(void)
    assert part.get_index() == 17

    # int32_t icu::MessagePattern::Part::getLength(void)
    assert part.get_length() == 1

    # int32_t icu::MessagePattern::Part::getLimit()
    assert part.get_limit() == 18

    # UMessagePatternPartType icu::MessagePattern::Part::getType()
    assert (part.get_type()
            == UMessagePatternPartType.UMSGPAT_PART_TYPE_ARG_START)

    # int32_t icu::MessagePattern::Part::getValue()
    assert part.get_value() == 4

    # int32_t icu::MessagePattern::Part::hashCode(void)
    assert part.hash_code()

    # static UBool icu::MessagePattern::Part::hasNumericValue(
    #       UMessagePatternPartType type
    # )
    assert not MessagePattern.Part.has_numeric_value(
        UMessagePatternPartType.UMSGPAT_PART_TYPE_ARG_START)

    assert MessagePattern.Part.has_numeric_value(
        UMessagePatternPartType.UMSGPAT_PART_TYPE_ARG_INT)



import copy

import pytest
from icupy import (
    ICUError, Locale, StringEnumeration, Transliterator, UErrorCode,
    UnicodeSet, UnicodeString, UParseError, UTransDirection, UTransPosition,
)


def test_adopt_filter():
    id1 = UnicodeString("Halfwidth-Fullwidth", -1)
    test1 = Transliterator.create_instance(
        id1,
        UTransDirection.UTRANS_FORWARD)
    src = UnicodeString(
        "123"
        "abc"
        "\uFF71\uFF72\uFF73\uFF74\uFF75",
        -1)

    text = src.clone()
    test1.transliterate(text)
    assert (text == "\uFF11\uFF12\uFF13"
                    "\uFF41\uFF42\uFF43"
                    "\u30A2\u30A4\u30A6\u30A8\u30AA")

    # void icu::Transliterator::adoptFilter(UnicodeFilter *adoptedFilter)
    uniset = UnicodeSet(UnicodeString("[^0-9]", -1))
    test1.adopt_filter(uniset)
    text = src.clone()
    test1.transliterate(text)
    assert (text == "123"
                    "\uFF41\uFF42\uFF43"
                    "\u30A2\u30A4\u30A6\u30A8\u30AA")

    test1.adopt_filter(None)
    text = src.clone()
    test1.transliterate(text)
    assert (text == "\uFF11\uFF12\uFF13"
                    "\uFF41\uFF42\uFF43"
                    "\u30A2\u30A4\u30A6\u30A8\u30AA")


def test_api():
    id4 = UnicodeString("Katakana-Latin", -1)
    trans = Transliterator.create_instance(
        id4,
        UTransDirection.UTRANS_FORWARD)

    # int32_t icu::Transliterator::countElements()
    assert trans.count_elements() == 3

    # const Transliterator &icu::Transliterator::getElement(
    #       int32_t index,
    #       UErrorCode &ec
    # )
    t0 = trans.get_element(0)
    assert isinstance(t0, Transliterator)

    t1 = trans.get_element(1)
    assert isinstance(t1, Transliterator)

    t2 = trans.get_element(2)
    assert isinstance(t2, Transliterator)

    with pytest.raises(ICUError) as exc_info:
        _ = trans.get_element(3)
    assert exc_info.value.args[0] == UErrorCode.U_INDEX_OUTOFBOUNDS_ERROR

    # const UnicodeFilter *icu::Transliterator::getFilter(void)
    uniset = trans.get_filter()
    assert isinstance(uniset, UnicodeSet)
    assert len(uniset) > 0

    # const UnicodeString &icu::Transliterator::getID(void)
    result = trans.get_id()
    assert isinstance(result, UnicodeString)
    assert result == id4

    # int32_t icu::Transliterator::getMaximumContextLength(void)
    assert trans.get_maximum_context_length() == 3

    # UnicodeSet &icu::Transliterator::getSourceSet(UnicodeSet &result)
    source = UnicodeSet()
    result = trans.get_source_set(source)
    assert isinstance(result, UnicodeSet)
    assert id(result) == id(source)
    assert len(source) > 0

    # UnicodeSet &icu::Transliterator::getTargetSet(UnicodeSet &result)
    target = UnicodeSet()
    result = trans.get_target_set(target)
    assert isinstance(result, UnicodeSet)
    assert id(result) == id(target)
    assert len(target) > 0

    # UnicodeFilter *icu::Transliterator::orphanFilter(void)
    uniset2 = trans.orphan_filter()
    assert isinstance(uniset2, UnicodeSet)
    assert len(uniset2) > 0

    # UnicodeString &icu::Transliterator::toRules(
    #       UnicodeString &result,
    #       UBool escapeUnprintable
    # )
    rules = UnicodeString()
    result = trans.to_rules(rules, False)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(rules)
    assert len(rules) > 0


def test_clone():
    id1 = UnicodeString("NFD;Jamo-Latin;Latin-Greek", -1)
    test1 = Transliterator.create_instance(
        id1,
        UTransDirection.UTRANS_FORWARD)

    # Transliterator *icu::Transliterator::clone()
    test2 = test1.clone()
    assert isinstance(test2, Transliterator)
    assert test2.get_id() == test1.get_id()

    test3 = copy.copy(test1)
    assert test3.get_id() == test1.get_id()

    test4 = copy.deepcopy(test1)
    assert test4.get_id() == test1.get_id()


def test_create_instance():
    # static Transliterator *icu::Transliterator::createFromRules(
    #       const UnicodeString &ID,
    #       const UnicodeString &rules,
    #       UTransDirection dir,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    id3 = UnicodeString("<ID>", -1)
    rules3 = UnicodeString(
        "pre {alpha} post > | @ ALPHA ;"
        "eALPHA > beta ;"
        "pre {beta} post > BETA @@ | ;"
        "post > xyz",
        -1)
    parse_error = UParseError()
    test3 = Transliterator.create_from_rules(
        id3,
        rules3,
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test3, Transliterator)
    assert test3.get_id() == id3
    parsed_rules3 = UnicodeString()
    test3.to_rules(parsed_rules3, True)

    test3a = Transliterator.create_from_rules(
        str(id3),
        rules3,
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test3a, Transliterator)
    assert test3a.get_id() == id3
    rules = UnicodeString()
    assert test3a.to_rules(rules, True) == parsed_rules3

    test3b = Transliterator.create_from_rules(
        id3,
        str(rules3),
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test3b, Transliterator)
    assert test3b.get_id() == id3
    rules = UnicodeString()
    assert test3b.to_rules(rules, True) == parsed_rules3

    test3c = Transliterator.create_from_rules(
        str(id3),
        str(rules3),
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test3c, Transliterator)
    assert test3c.get_id() == id3
    rules = UnicodeString()
    assert test3c.to_rules(rules, True) == parsed_rules3

    # [1]
    # static Transliterator *icu::Transliterator::createInstance(
    #       const UnicodeString &ID,
    #       UTransDirection dir,
    #       UErrorCode &status
    # )
    id1 = UnicodeString("NFD;Jamo-Latin;Latin-Greek", -1)
    test1 = Transliterator.create_instance(
        id1,
        UTransDirection.UTRANS_FORWARD)
    assert isinstance(test1, Transliterator)
    assert test1.get_id() == id1
    parsed_rules1 = UnicodeString()
    test1.to_rules(parsed_rules1, True)

    test1a = Transliterator.create_instance(
        str(id1),
        UTransDirection.UTRANS_FORWARD)
    assert isinstance(test1a, Transliterator)
    assert test1a.get_id() == id1
    rules = UnicodeString()
    assert test1a.to_rules(rules, True) == parsed_rules1

    # [2]
    # static Transliterator *icu::Transliterator::createInstance(
    #       const UnicodeString &ID,
    #       UTransDirection dir,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    parse_error = UParseError()
    test2 = Transliterator.create_instance(
        id1,
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test2, Transliterator)
    assert test2.get_id() == id1
    parsed_rules1 = UnicodeString()
    test2.to_rules(parsed_rules1, True)

    test2a = Transliterator.create_instance(
        str(id1),
        UTransDirection.UTRANS_FORWARD,
        parse_error)
    assert isinstance(test2a, Transliterator)
    assert test2a.get_id() == id1
    rules = UnicodeString()
    assert test2a.to_rules(rules, True) == parsed_rules1

    # Transliterator *icu::Transliterator::createInverse(UErrorCode &status)
    id4 = UnicodeString("Hiragana-Katakana", -1)
    test4 = Transliterator.create_instance(
        id4,
        UTransDirection.UTRANS_FORWARD)
    test5 = test4.create_inverse()
    assert isinstance(test5, Transliterator)
    assert test5.get_id() == "Katakana-Hiragana"


def test_get_available_ids():
    # static StringEnumeration *icu::Transliterator::getAvailableIDs(
    #       UErrorCode &ec
    # )
    ids = Transliterator.get_available_ids()
    assert isinstance(ids, StringEnumeration)
    assert len(ids) > 0
    assert "Hiragana-Katakana" in ids


def test_get_available_variant():
    source = UnicodeString()
    target = UnicodeString()
    variant = UnicodeString()
    count = 0

    # static int32_t icu::Transliterator::countAvailableSources(void)
    for i in range(Transliterator.count_available_sources()):
        # static UnicodeString &icu::Transliterator::getAvailableSource(
        #       int32_t index,
        #       UnicodeString &result
        # )
        result = Transliterator.get_available_source(i, source)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(source)
        assert len(source) > 0

        # static int32_t icu::Transliterator::countAvailableTargets(
        #       const UnicodeString &source
        # )
        targets = Transliterator.count_available_targets(source)
        assert targets == Transliterator.count_available_targets(str(source))
        for j in range(targets):
            # static UnicodeString &icu::Transliterator::getAvailableTarget(
            #       int32_t index,
            #       const UnicodeString &source,
            #       UnicodeString &result
            # )
            result = Transliterator.get_available_target(j, source, target)
            assert isinstance(result, UnicodeString)
            assert id(result) == id(target)
            assert len(target) > 0

            assert result == Transliterator.get_available_target(
                j, str(source), target)

            # static int32_t icu::Transliterator::countAvailableVariants(
            #       const UnicodeString &source,
            #       const UnicodeString &target
            # )
            variants = Transliterator.count_available_variants(
                source,
                target)
            assert variants == Transliterator.count_available_variants(
                str(source),
                target)
            assert variants == Transliterator.count_available_variants(
                source,
                str(target))
            assert variants == Transliterator.count_available_variants(
                str(source),
                str(target))

            for k in range(variants):
                # static UnicodeString &
                # icu::Transliterator::getAvailableVariant(
                #       int32_t index,
                #       const UnicodeString &source,
                #       const UnicodeString &target,
                #       UnicodeString &result
                # )
                result = Transliterator.get_available_variant(
                    k,
                    source,
                    target,
                    variant)
                assert isinstance(result, UnicodeString)
                assert id(result) == id(variant)
                assert len(variant) >= 0

                assert result == Transliterator.get_available_variant(
                    k,
                    str(source),
                    target,
                    variant)
                assert result == Transliterator.get_available_variant(
                    k,
                    source,
                    str(target),
                    variant)
                assert result == Transliterator.get_available_variant(
                    k,
                    str(source),
                    str(target),
                    variant)
                count += 1
    assert count > 0


def test_get_display_name():
    # [1]
    # static UnicodeString &icu::Transliterator::getDisplayName(
    #       const UnicodeString &ID,
    #       const Locale &inLocale,
    #       UnicodeString &result
    # )
    display_name = UnicodeString()
    result = Transliterator.get_display_name(
        UnicodeString("Hiragana-Katakana", -1),
        Locale.get_us(),
        display_name)
    assert result == display_name
    assert result == "Hiragana to Katakana"

    display_name.remove()
    result = Transliterator.get_display_name(
        "Hiragana-Katakana",
        Locale.get_us(),
        display_name)
    assert result == display_name
    assert result == "Hiragana to Katakana"

    display_name.remove()
    result = Transliterator.get_display_name(
        UnicodeString("Hiragana-Katakana", -1),
        "en_US",
        display_name)
    assert result == display_name
    assert result == "Hiragana to Katakana"

    display_name.remove()
    result = Transliterator.get_display_name(
        "Hiragana-Katakana",
        "en_US",
        display_name)
    assert result == display_name
    assert result == "Hiragana to Katakana"

    # [2]
    # static UnicodeString &icu::Transliterator::getDisplayName(
    #       const UnicodeString &ID,
    #       UnicodeString &result
    # )
    result = Transliterator.get_display_name(
        UnicodeString("Hiragana-Katakana", -1),
        display_name)
    assert result == display_name
    assert len(result) > 0

    display_name.remove()
    result = Transliterator.get_display_name(
        "Hiragana-Katakana",
        display_name)
    assert result == display_name
    assert len(result) > 0


def test_register_instance():
    id3 = UnicodeString("Any-MyTransliterator", -1)
    rules3 = UnicodeString(
        "pre {alpha} post > | @ ALPHA ;"
        "eALPHA > beta ;"
        "pre {beta} post > BETA @@ | ;"
        "post > xyz",
        -1)
    parse_error = UParseError()
    test3 = Transliterator.create_from_rules(
        id3,
        rules3,
        UTransDirection.UTRANS_FORWARD,
        parse_error)

    ids = Transliterator.get_available_ids()
    assert str(id3) not in ids

    # static void icu::Transliterator::registerInstance(
    #       Transliterator *adoptedObj
    # )
    Transliterator.register_instance(test3)

    # static void icu::Transliterator::registerAlias(
    #       const UnicodeString &aliasID,
    #       const UnicodeString &realID
    # )
    id3a = UnicodeString("Any-MyRule", -1)
    Transliterator.register_alias(id3a, id3)

    ids = Transliterator.get_available_ids()
    assert str(id3) in ids
    assert str(id3a) in ids

    test3a = Transliterator.create_instance(
        id3a,
        UTransDirection.UTRANS_FORWARD)
    assert test3a.get_id() == id3a

    text = UnicodeString("prealphapost prebetapost", -1)
    test3a.transliterate(text)
    assert text == "prbetaxyz preBETApost"

    # static void icu::Transliterator::unregister(const UnicodeString &ID)
    Transliterator.unregister(id3)
    Transliterator.unregister(id3a)
    ids = Transliterator.get_available_ids()
    assert str(id3) not in ids
    assert str(id3a) not in ids

    Transliterator.register_instance(test3)
    Transliterator.register_alias("Any-MyRule1", id3)
    Transliterator.register_alias(UnicodeString("Any-MyRule2"), str(id3))
    Transliterator.register_alias("Any-MyRule3", str(id3))
    ids = Transliterator.get_available_ids()
    assert "Any-MyRule1" in ids
    assert "Any-MyRule2" in ids
    assert "Any-MyRule3" in ids


def test_transliterate():
    id4 = UnicodeString("Hiragana-Katakana", -1)
    test4 = Transliterator.create_instance(
        id4,
        UTransDirection.UTRANS_FORWARD)
    src = UnicodeString("\u3042\u3044\u3046\u3048\u304A", -1)

    # [1]
    # void icu::Transliterator::transliterate(Replaceable &text)
    text = src.clone()
    test4.transliterate(text)
    assert text == "\u30A2\u30A4\u30A6\u30A8\u30AA"

    # [2]
    # int32_t icu::Transliterator::transliterate(
    #       Replaceable &text,
    #       int32_t start,
    #       int32_t limit
    # )
    text = src.clone()
    result = test4.transliterate(text, 1, 3)
    assert text == "\u3042\u30A4\u30A6\u3048\u304A"
    assert result == 3

    # [3]
    # void icu::Transliterator::transliterate(
    #       Replaceable &text,
    #       UTransPosition &index,
    #       const UnicodeString &insertion,
    #       UErrorCode &status
    # )
    text = src.clone()
    index = UTransPosition()
    index.context_start = 0
    index.context_limit = 5
    index.start = 1
    index.limit = 4
    insertion = UnicodeString("_", -1)
    test4.transliterate(text, index, insertion)
    assert index.context_start == 0
    assert index.context_limit == 6
    assert index.start == 5
    assert index.limit == 5
    assert text == "\u3042\u30A4\u30A6\u30A8_\u304A"

    text = src.clone()
    index = UTransPosition(0, 5, 1, 4)
    test4.transliterate(text, index, "_")
    assert index.context_start == 0
    assert index.context_limit == 6
    assert index.start == 5
    assert index.limit == 5
    assert text == "\u3042\u30A4\u30A6\u30A8_\u304A"

    # [4]
    # void icu::Transliterator::transliterate(
    #       Replaceable &text,
    #       UTransPosition &index,
    #       UChar32 insertion,
    #       UErrorCode &status
    # )
    text = src.clone()
    index = UTransPosition(0, 5, 1, 4)
    assert index.context_start == 0
    assert index.context_limit == 5
    assert index.start == 1
    assert index.limit == 4
    insertion = ord("_")
    test4.transliterate(text, index, insertion)
    assert index.context_start == 0
    assert index.context_limit == 6
    assert index.start == 5
    assert index.limit == 5
    assert text == "\u3042\u30A4\u30A6\u30A8_\u304A"

    # [5]
    # void icu::Transliterator::transliterate(
    #       Replaceable &text,
    #       UTransPosition &index,
    #       UErrorCode &status
    # )
    text = src.clone()
    # UTransPosition(context_start, start, limit, context_limit)
    index = UTransPosition(0, 5, 1, 4)
    assert index.context_start == 0
    assert index.context_limit == 5
    assert index.start == 1
    assert index.limit == 4
    test4.transliterate(text, index)
    assert index.context_start == 0
    assert index.context_limit == 5
    assert index.start == 4
    assert index.limit == 4
    assert text == "\u3042\u30A4\u30A6\u30A8\u304A"



import os
import sys
from contextlib import contextmanager
from pathlib import Path


def _startup():
    path = str((Path(__file__).parent / ".." / "src").resolve())
    if path not in sys.path:
        sys.path.append(path)

    if sys.platform.startswith("win"):
        path = os.path.join(os.getenv("ICU_ROOT", "C:\\icu"),
                            "bin" + "64" if sys.maxsize > 2 ** 32 else "")
        assert os.path.isdir(path)  # ICU4C not found
        os.add_dll_directory(path)


@contextmanager
def gc(obj, closer):
    assert callable(closer)
    try:
        yield obj
    finally:
        closer(obj)


_startup()



import copy

import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 50:
    pytest.skip("ICU4C<50", allow_module_level=True)

from icupy import (
    Locale, StringEnumeration, TimeZoneNames, UTimeZoneNameType,
    UnicodeString,
)


def test_api():
    # static TimeZoneNames *icu::TimeZoneNames::createInstance(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    tzn1 = TimeZoneNames.create_instance(Locale("en"))
    assert isinstance(tzn1, TimeZoneNames)

    tzn2 = TimeZoneNames.create_instance("en")
    assert isinstance(tzn2, TimeZoneNames)

    tzn3 = TimeZoneNames.create_instance("ja")

    # UBool icu::TimeZoneNames::operator!=(const TimeZoneNames &other)
    assert not (tzn1 != tzn2)
    assert tzn1 != tzn3
    assert tzn2 != tzn3

    # UBool icu::TimeZoneNames::operator==(const TimeZoneNames &other)
    assert tzn1 == tzn2
    assert not (tzn1 == tzn3)
    assert not (tzn2 == tzn3)

    # [1]
    # StringEnumeration *icu::TimeZoneNames::getAvailableMetaZoneIDs(
    #       const UnicodeString &tzID,
    #       UErrorCode &status
    # )
    it1 = tzn1.get_available_meta_zone_ids(
        UnicodeString("America/Los_Angeles"))
    assert isinstance(it1, StringEnumeration)
    assert len(it1) > 0
    assert "America_Pacific" in it1

    it2 = tzn1.get_available_meta_zone_ids(
        "America/Los_Angeles")
    assert isinstance(it2, StringEnumeration)
    assert it2 == it1

    # [2]
    # StringEnumeration *icu::TimeZoneNames::getAvailableMetaZoneIDs(
    #       UErrorCode &status
    # )
    it3 = tzn1.get_available_meta_zone_ids()
    assert isinstance(it3, StringEnumeration)
    assert len(it3) > 0
    assert "America_Pacific" in it3

    date = 1358208000000.0  # 2013-01-15T00:00:00Z
    name = UnicodeString()

    # UnicodeString &icu::TimeZoneNames::getDisplayName(
    #       const UnicodeString &tzID,
    #       UTimeZoneNameType type,
    #       UDate date,
    #       UnicodeString &name
    # )
    result = tzn1.get_display_name(
        UnicodeString("America/Los_Angeles"),
        UTimeZoneNameType.UTZNM_LONG_STANDARD,
        date,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Pacific Standard Time"

    result = tzn1.get_display_name(
        "America/Los_Angeles",
        UTimeZoneNameType.UTZNM_LONG_DAYLIGHT,
        date,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Pacific Daylight Time"

    # UnicodeString &icu::TimeZoneNames::getExemplarLocationName(
    #       const UnicodeString &tzID,
    #       UnicodeString &name
    # )
    result = tzn1.get_exemplar_location_name(
        UnicodeString("America/Los_Angeles"),
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Los Angeles"

    result = tzn1.get_exemplar_location_name(
        "Asia/Tokyo",
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Tokyo"

    # UnicodeString &icu::TimeZoneNames::getMetaZoneDisplayName(
    #       const UnicodeString &mzID,
    #       UTimeZoneNameType type,
    #       UnicodeString &name
    # )
    result = tzn1.get_meta_zone_display_name(
        UnicodeString("America_Pacific"),
        UTimeZoneNameType.UTZNM_LONG_STANDARD,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Pacific Standard Time"

    result = tzn1.get_meta_zone_display_name(
        "America_Pacific",
        UTimeZoneNameType.UTZNM_LONG_DAYLIGHT,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result == "Pacific Daylight Time"

    mz_id = UnicodeString()

    # UnicodeString &icu::TimeZoneNames::getMetaZoneID(
    #       const UnicodeString &tzID,
    #       UDate date,
    #       UnicodeString &mzID
    # )
    result = tzn1.get_meta_zone_id(
        UnicodeString("America/Los_Angeles"),
        date,
        mz_id)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(mz_id)
    assert result == "America_Pacific"

    result = tzn1.get_meta_zone_id(
        "America/Los_Angeles",
        date,
        mz_id)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(mz_id)
    assert result == "America_Pacific"

    tz_id = UnicodeString()

    # UnicodeString &icu::TimeZoneNames::getReferenceZoneID(
    #       const UnicodeString &mzID,
    #       const char *region,
    #       UnicodeString &tzID
    # )
    result = tzn1.get_reference_zone_id(
        UnicodeString("America_Pacific"),
        "001",
        tz_id)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(tz_id)
    assert result == "America/Los_Angeles"

    result = tzn1.get_reference_zone_id(
        "America_Pacific",
        "001",
        tz_id)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(tz_id)
    assert result == "America/Los_Angeles"

    # UnicodeString &icu::TimeZoneNames::getTimeZoneDisplayName(
    #       const UnicodeString &tzID,
    #       UTimeZoneNameType type,
    #       UnicodeString &name
    # )
    result = tzn1.get_time_zone_display_name(
        UnicodeString("America/Los_Angeles"),
        UTimeZoneNameType.UTZNM_LONG_STANDARD,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result.is_bogus()

    result = tzn1.get_time_zone_display_name(
        "America/Los_Angeles",
        UTimeZoneNameType.UTZNM_LONG_STANDARD,
        name)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(name)
    assert result.is_bogus()


def test_clone():
    tzn1 = TimeZoneNames.create_instance(Locale("en"))

    # TimeZoneNames *icu::TimeZoneNames::clone()
    tzn2 = tzn1.clone()
    assert isinstance(tzn2, TimeZoneNames)
    assert tzn2 == tzn1

    tzn3 = copy.copy(tzn1)
    assert tzn3 == tzn1

    tzn4 = copy.deepcopy(tzn1)
    assert tzn4 == tzn1


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_create_tzdb_instance():
    # static TimeZoneNames *icu::TimeZoneNames::createTZDBInstance(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    tzn1 = TimeZoneNames.create_tzdb_instance(Locale("en"))
    assert isinstance(tzn1, TimeZoneNames)

    tzn2 = TimeZoneNames.create_tzdb_instance("en")
    assert isinstance(tzn2, TimeZoneNames)

    # UBool icu::TZDBTimeZoneNames::operator==(...) is not implemented (ICU 69)
    # assert tzn1 == tzn2



import os
import subprocess
import sys

from setuptools import Extension
from setuptools.command.build_ext import build_ext

# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
        extdir = os.path.join(extdir, ext.name)

        # required for auto-detection of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        env = os.environ.copy()
        cmake_build_type = env.get("CMAKE_BUILD_TYPE", "Release")
        cfg = "Debug" if self.debug else cmake_build_type

        env["CXXFLAGS"] = " ".join([
            env.get("CXXFLAGS", "").strip(),
            "-DVERSION_INFO=\\\"{}\\\"".format(
                self.distribution.get_version()),
        ]).strip()

        cmake_args = [
            "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=" + extdir,
            "-DPYTHON_EXECUTABLE=" + sys.executable,
            "-DCMAKE_BUILD_TYPE=" + cfg,
        ]

        build_args = []

        if sys.platform.startswith("win"):
            if "ICU_ROOT" not in env:
                env["ICU_ROOT"] = "C:\\icu"
                self.warn(
                    "ICU_ROOT environment variable was not set. "
                    "Using default '{}'.".format(env["ICU_ROOT"]))
            cmake_args += [
                "-Wno-dev",
            ]

        cmake_generator = env.get("CMAKE_GENERATOR")
        is_msbuild = (
                self.compiler.compiler_type == "msvc"
                and (cmake_generator is None
                     or cmake_generator.startswith("Visual Studio"))
        )
        is_multi_config = (
                is_msbuild
                or cmake_generator in ["Ninja Multi-Config", "Xcode"]
        )

        if is_multi_config:
            cmake_args += [
                "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}".format(
                    cfg.upper(), extdir),
            ]
            build_args += [
                "--config", cfg,
            ]

        if is_msbuild:
            if "CMAKE_GENERATOR_PLATFORM" not in env:
                cmake_args += [
                    "-A", PLAT_TO_CMAKE[self.plat_name],
                ]

        if ("CMAKE_BUILD_PARALLEL_LEVEL" not in env
                and not is_msbuild
                and (cmake_generator is None
                     or not cmake_generator.startswith("Ninja"))):
            jobs = ((hasattr(self, "parallel") and self.parallel)
                    or os.cpu_count())
            if jobs:
                build_args += [
                    "--parallel", str(jobs),
                ]

        self.announce(
            "-- CXXFLAGS environment variable: " + repr(env.get("CXXFLAGS")),
            level=2)
        self.announce(
            "-- CMake environment variables: " + repr(
                ["{}={}".format(k, v) for k, v in env.items()
                 if k.upper().startswith("CMAKE")]),
            level=2)
        self.announce(
            "-- CMake build system options: " + repr(cmake_args),
            level=2)
        self.announce(
            "-- CMake build options: " + repr(build_args),
            level=2)

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)

        subprocess.check_call(
            ["cmake", ext.sourcedir] + cmake_args,
            cwd=self.build_temp,
            env=env)
        subprocess.check_call(
            ["cmake", "--build", "."] + build_args,
            cwd=self.build_temp,
            env=env)


def build(setup_kwargs):
    src = (setup_kwargs["package_dir"][""] if "package_dir" in setup_kwargs
           else "")
    ext_modules = [CMakeExtension(setup_kwargs["name"], src)]
    cmdclass = dict(build_ext=CMakeBuild)
    setup_kwargs.update({
        "ext_modules": ext_modules,
        "cmdclass": cmdclass,
    })



import pytest
from icupy import (
    DecimalFormat, Format, Locale, NumberFormat, UNumberFormatStyle,
    U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


def test_api():
    assert issubclass(NumberFormat, Format)
    assert issubclass(DecimalFormat, NumberFormat)

    # [1]
    # static const Locale *icu::NumberFormat::getAvailableLocales(
    #       int32_t &count
    # )
    it = NumberFormat.get_available_locales()
    assert isinstance(it, list)
    assert len(it) > 0
    assert all(isinstance(x, Locale) for x in it)

    fmt = NumberFormat.create_currency_instance(Locale.get_us())

    # const char16_t *icu::NumberFormat::getCurrency()
    result = fmt.get_currency()
    assert isinstance(result, str)
    assert result == "USD"

    # int32_t icu::NumberFormat::getMaximumFractionDigits(void)
    assert fmt.get_maximum_fraction_digits() == 2

    # int32_t icu::NumberFormat::getMaximumIntegerDigits(void)
    assert fmt.get_maximum_integer_digits() == 2e+9

    # int32_t icu::NumberFormat::getMinimumFractionDigits(void)
    assert fmt.get_minimum_fraction_digits() == 2

    # int32_t icu::NumberFormat::getMinimumIntegerDigits(void)
    assert fmt.get_minimum_integer_digits() == 1

    # UBool icu::NumberFormat::isGroupingUsed(void)
    assert fmt.is_grouping_used()

    # UBool icu::NumberFormat::isParseIntegerOnly(void)
    assert not fmt.is_parse_integer_only()


def test_create_currency_instance():
    default_locale = Locale.get_default()

    try:
        in_locale = Locale.get_us()
        Locale.set_default(in_locale)

        # [1]
        # static NumberFormat *icu::NumberFormat::createCurrencyInstance(
        #       const Locale &inLocale,
        #       UErrorCode &
        # )
        fmt1 = NumberFormat.create_currency_instance(in_locale)
        assert isinstance(fmt1, DecimalFormat)

        fmt1a = NumberFormat.create_currency_instance("en-US")
        assert isinstance(fmt1a, DecimalFormat)
        assert fmt1 == fmt1a

        append_to = UnicodeString()
        assert fmt1.format(1000, append_to) == "$1,000.00"

        # [2]
        # static NumberFormat *icu::NumberFormat::createCurrencyInstance(
        #       UErrorCode &
        # )
        fmt2 = NumberFormat.create_currency_instance()
        assert isinstance(fmt2, DecimalFormat)
        assert fmt1 == fmt2

        append_to.remove()
        assert fmt2.format(1000, append_to) == "$1,000.00"
    finally:
        Locale.set_default(default_locale)


def test_create_instance():
    default_locale = Locale.get_default()

    try:
        in_locale = Locale.get_us()
        Locale.set_default(in_locale)

        # [1]
        # static NumberFormat *icu::NumberFormat::createInstance(
        #       const Locale &desiredLocale,
        #       UNumberFormatStyle style,
        #       UErrorCode &errorCode
        # )
        fmt1 = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_DEFAULT)
        assert isinstance(fmt1, DecimalFormat)

        fmt1a = NumberFormat.create_instance(
            "en-US",
            UNumberFormatStyle.UNUM_DEFAULT)
        assert isinstance(fmt1a, DecimalFormat)
        assert fmt1 == fmt1a

        append_to = UnicodeString()
        assert fmt1.format(1000, append_to) == "1,000"

        # UNUM_PATTERN_DECIMAL -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_PATTERN_DECIMAL)

        # UNUM_DECIMAL
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_DECIMAL)
        assert isinstance(test, DecimalFormat)

        # UNUM_CURRENCY
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_CURRENCY)
        assert isinstance(test, DecimalFormat)

        # UNUM_PERCENT
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_PERCENT)
        assert isinstance(test, DecimalFormat)

        # UNUM_SCIENTIFIC
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_SCIENTIFIC)
        assert isinstance(test, DecimalFormat)

        # UNUM_SPELLOUT -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_SPELLOUT)

        # UNUM_ORDINAL -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_ORDINAL)

        # UNUM_DURATION -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_DURATION)

        # UNUM_NUMBERING_SYSTEM -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_NUMBERING_SYSTEM)

        # UNUM_PATTERN_RULEBASED -> U_UNSUPPORTED_ERROR
        # test = NumberFormat.create_instance(
        #     in_locale,
        #     UNumberFormatStyle.UNUM_PATTERN_RULEBASED)

        # UNUM_CURRENCY_ISO
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_CURRENCY_ISO)
        assert isinstance(test, DecimalFormat)

        # UNUM_CURRENCY_PLURAL
        test = NumberFormat.create_instance(
            in_locale,
            UNumberFormatStyle.UNUM_CURRENCY_PLURAL)
        assert isinstance(test, DecimalFormat)

        # [2]
        # static NumberFormat *icu::NumberFormat::createInstance(
        #       const Locale &inLocale,
        #       UErrorCode &
        # )
        fmt2 = NumberFormat.create_instance(in_locale)
        assert isinstance(fmt2, DecimalFormat)

        fmt2a = NumberFormat.create_instance("en-US")
        assert isinstance(fmt2a, DecimalFormat)
        assert fmt2 == fmt2a

        append_to.remove()
        assert fmt2.format(1000, append_to) == "1,000"

        # [3]
        # static NumberFormat *icu::NumberFormat::createInstance(UErrorCode &)
        fmt3 = NumberFormat.create_instance()
        assert isinstance(fmt3, DecimalFormat)
        assert fmt1 == fmt2 == fmt3

        append_to.remove()
        assert fmt3.format(1000, append_to) == "1,000"
    finally:
        Locale.set_default(default_locale)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 53, reason="ICU4C<53")
def test_create_instance_53():
    # UNUM_CURRENCY_ACCOUNTING
    test = NumberFormat.create_instance(
        Locale.get_us(),
        UNumberFormatStyle.UNUM_CURRENCY_ACCOUNTING)
    assert isinstance(test, DecimalFormat)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_create_instance_54():
    # UNUM_CASH_CURRENCY
    test = NumberFormat.create_instance(
        Locale.get_us(),
        UNumberFormatStyle.UNUM_CASH_CURRENCY)
    assert isinstance(test, DecimalFormat)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 56, reason="ICU4C<56")
def test_create_instance_56():
    # UNUM_DECIMAL_COMPACT_SHORT -> U_UNSUPPORTED_ERROR
    # test = NumberFormat.create_instance(
    #     Locale.get_us(),
    #     UNumberFormatStyle.UNUM_DECIMAL_COMPACT_SHORT)

    # UNUM_DECIMAL_COMPACT_LONG -> U_UNSUPPORTED_ERROR
    # test = NumberFormat.create_instance(
    #     Locale.get_us(),
    #     UNumberFormatStyle.UNUM_DECIMAL_COMPACT_LONG)

    # UNUM_CURRENCY_STANDARD
    test = NumberFormat.create_instance(
        Locale.get_us(),
        UNumberFormatStyle.UNUM_CURRENCY_STANDARD)
    assert isinstance(test, DecimalFormat)


def test_create_percent_instance():
    default_locale = Locale.get_default()

    try:
        in_locale = Locale.get_us()
        Locale.set_default(in_locale)

        # [1]
        # static NumberFormat *icu::NumberFormat::createPercentInstance(
        #       const Locale &inLocale,
        #       UErrorCode &
        # )
        fmt1 = NumberFormat.create_percent_instance(in_locale)
        assert isinstance(fmt1, DecimalFormat)

        fmt1a = NumberFormat.create_percent_instance("en-US")
        assert isinstance(fmt1a, DecimalFormat)
        assert fmt1 == fmt1a

        append_to = UnicodeString()
        assert fmt1.format(0.8, append_to) == "80%"

        # [2]
        # static NumberFormat *icu::NumberFormat::createPercentInstance(
        #       UErrorCode &
        # )
        fmt2 = NumberFormat.create_percent_instance()
        assert isinstance(fmt2, DecimalFormat)
        assert fmt1 == fmt2

        append_to.remove()
        assert fmt2.format(0.8, append_to) == "80%"
    finally:
        Locale.set_default(default_locale)


def test_create_scientific_instance():
    default_locale = Locale.get_default()

    try:
        in_locale = Locale.get_us()
        Locale.set_default(in_locale)

        # [1]
        # static NumberFormat *icu::NumberFormat::createScientificInstance(
        #       const Locale &inLocale,
        #       UErrorCode &
        # )
        fmt1 = NumberFormat.create_scientific_instance(in_locale)
        assert isinstance(fmt1, DecimalFormat)

        fmt1a = NumberFormat.create_scientific_instance("en-US")
        assert isinstance(fmt1a, DecimalFormat)
        assert fmt1 == fmt1a

        append_to = UnicodeString()
        assert fmt1.format(1000, append_to) == "1E3"

        # [2]
        # static NumberFormat *icu::NumberFormat::createScientificInstance(
        #       UErrorCode &
        # )
        fmt2 = NumberFormat.create_scientific_instance()
        assert isinstance(fmt2, DecimalFormat)
        assert fmt1 == fmt2

        append_to.remove()
        assert fmt2.format(1000, append_to) == "1E3"
    finally:
        Locale.set_default(default_locale)



import copy

import pytest
from icupy import (
    Calendar, CurrencyAmount, DateInterval, Formattable, ICUError,
    INT32_MAX, TimeUnit, TimeUnitAmount, TimeZone, UErrorCode, UnicodeString,
)


def test_api():
    fmt2 = Formattable(1231027200000.0, Formattable.IS_DATE)
    fmt3 = Formattable(float(INT32_MAX + 1))
    fmt4 = Formattable(INT32_MAX)
    fmt5 = Formattable(INT32_MAX + 1)
    fmt8 = Formattable(UnicodeString("1e-3"))
    array = [fmt2, fmt3, fmt4, fmt5, fmt8]
    fmt10 = Formattable(array)

    # Type icu::Formattable::getType(void)
    assert fmt2.get_type() == Formattable.DATE
    assert fmt3.get_type() == Formattable.DOUBLE
    assert fmt4.get_type() == Formattable.LONG
    assert fmt5.get_type() == Formattable.INT64
    assert fmt8.get_type() == Formattable.STRING
    assert fmt10.get_type() == Formattable.ARRAY

    # [2]
    # const Formattable *icu::Formattable::getArray(
    #       int32_t &count,
    #       UErrorCode &status
    # )
    array2 = fmt10.get_array()
    assert isinstance(array2, list)
    assert array2 == array

    with pytest.raises(ICUError) as exc_info:
        _ = fmt2.get_array()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # UDate icu::Formattable::getDate(UErrorCode &status)
    assert fmt2.get_date() == 1231027200000.0

    with pytest.raises(ICUError) as exc_info:
        _ = fmt3.get_date()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # StringPiece icu::Formattable::getDecimalNumber(UErrorCode &status)
    result = fmt3.get_decimal_number()
    assert isinstance(result, str)
    assert result == "2.147483648E+9"

    with pytest.raises(ICUError) as exc_info:
        _ = fmt2.get_decimal_number()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_STATE_ERROR

    # double icu::Formattable::getDouble(UErrorCode &status)
    assert fmt3.get_double() == INT32_MAX + 1
    assert fmt4.get_double() == INT32_MAX
    assert fmt5.get_double() == INT32_MAX + 1

    with pytest.raises(ICUError) as exc_info:
        _ = fmt2.get_double()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # int64_t icu::Formattable::getInt64(UErrorCode &status)
    assert fmt3.get_int64() == INT32_MAX + 1
    assert fmt4.get_int64() == INT32_MAX
    assert fmt5.get_int64() == INT32_MAX + 1

    with pytest.raises(ICUError) as exc_info:
        _ = fmt2.get_int64()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # int32_t icu::Formattable::getLong(UErrorCode &status)
    assert fmt4.get_long() == INT32_MAX

    with pytest.raises(ICUError) as exc_info:
        _ = fmt5.get_long()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # const UObject *icu::Formattable::getObject()
    assert fmt2.get_object() is None
    assert fmt10.get_object() is None

    # [1]
    # UnicodeString &icu::Formattable::getString(UErrorCode &status)
    result = fmt8.get_string()
    assert isinstance(result, UnicodeString)
    assert result == "1e-3"

    # [4]
    # UnicodeString &icu::Formattable::getString(
    #       UnicodeString &result,
    #       UErrorCode &status
    # )
    result = UnicodeString()
    output = fmt8.get_string(result)
    assert isinstance(output, UnicodeString)
    assert id(result) == id(output)
    assert output == result == "1e-3"

    with pytest.raises(ICUError) as exc_info:
        _ = fmt2.get_string()
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    # UBool icu::Formattable::isNumeric()
    assert not fmt2.is_numeric()
    assert fmt3.is_numeric()
    assert fmt4.is_numeric()
    assert fmt5.is_numeric()
    assert not fmt8.is_numeric()
    assert not fmt10.is_numeric()

    # UBool icu::Formattable::operator!=(const Formattable &other)
    assert fmt3 != fmt4
    assert fmt3 != fmt5

    # UBool icu::Formattable::operator==(const Formattable &other)
    assert not (fmt3 == fmt4)
    assert not (fmt3 == fmt5)

    # Formattable &icu::Formattable::operator[](int32_t index)
    assert fmt10[0] == fmt2
    assert fmt10[1] == fmt3
    assert fmt10[2] == fmt4
    assert fmt10[3] == fmt5
    assert fmt10[4] == fmt8
    with pytest.raises(IndexError):  # Non-standard
        _ = fmt10[5]
    assert fmt10[-1] == fmt8  # Non-standard

    with pytest.raises(ICUError) as exc_info:  # Non-standard
        _ = fmt2[0]
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    fmt1 = Formattable()

    # void icu::Formattable::setArray(
    #       const Formattable *array,
    #       int32_t count
    # )
    fmt1.set_array(array)
    assert fmt1.get_type() == Formattable.ARRAY
    assert fmt1.get_array() == array
    assert fmt1 == fmt10

    # void icu::Formattable::setDate(UDate d)
    fmt1.set_date(1231027200000.0)
    assert fmt1.get_type() == Formattable.DATE
    assert fmt1.get_date() == 1231027200000.0
    assert fmt1 == fmt2

    # void icu::Formattable::setDecimalNumber(
    #       StringPiece numberString,
    #       UErrorCode &status
    # )
    fmt1.set_decimal_number("1e-3")
    assert fmt1.get_type() == Formattable.DOUBLE
    assert fmt1.get_double() == 1e-3

    # void icu::Formattable::setDouble(double d)
    fmt1.set_double(INT32_MAX + 1)
    assert fmt1.get_type() == Formattable.DOUBLE
    assert fmt1.get_double() == INT32_MAX + 1
    assert fmt1 == fmt3

    # void icu::Formattable::setInt64(int64_t ll)
    fmt1.set_int64(INT32_MAX + 1)
    assert fmt1.get_type() == Formattable.INT64
    assert fmt1.get_int64() == INT32_MAX + 1
    assert fmt1 == fmt5

    # void icu::Formattable::setLong(int32_t l)
    fmt1.set_long(INT32_MAX)
    assert fmt1.get_type() == Formattable.LONG
    assert fmt1.get_long() == INT32_MAX
    assert fmt1 == fmt4

    # void icu::Formattable::setString(const UnicodeString &stringToCopy)
    fmt1.set_string(UnicodeString("1e-3"))
    assert fmt1.get_type() == Formattable.STRING
    assert fmt1.get_string() == "1e-3"
    assert fmt1 == fmt8

    fmt1.set_string("1e+3")
    assert fmt1.get_type() == Formattable.STRING
    assert fmt1.get_string() == "1e+3"
    assert fmt1 != fmt8


def test_clone():
    fmt2 = Formattable(1231027200000.0, Formattable.IS_DATE)
    fmt3 = Formattable(float(INT32_MAX))
    fmt4 = Formattable(INT32_MAX)
    fmt5 = Formattable(INT32_MAX + 1)
    fmt8 = Formattable(UnicodeString("1e-3"))
    array = [fmt2, fmt3, fmt4, fmt5, fmt8]
    fmt10 = Formattable(array)

    # Formattable *icu::Formattable::clone()
    test1 = fmt10.clone()
    assert isinstance(test1, Formattable)
    assert test1.get_type() == Formattable.ARRAY
    assert test1.get_array() == array

    test2 = copy.copy(fmt10)
    assert test2.get_type() == Formattable.ARRAY
    assert test2.get_array() == array

    test3 = copy.deepcopy(fmt10)
    assert test3.get_type() == Formattable.ARRAY
    assert test3.get_array() == array


def test_formattable():
    # [1]
    # icu::Formattable::Formattable()
    fmt1 = Formattable()
    assert fmt1.get_type() == Formattable.LONG
    assert fmt1.get_long() == 0

    # [2]
    # icu::Formattable::Formattable(
    #       UDate d,
    #       ISDATE flag
    # )
    fmt2 = Formattable(1231027200000.0, Formattable.IS_DATE)
    assert fmt2.get_type() == Formattable.DATE
    assert fmt2.get_date() == 1231027200000.0

    # [3]
    # icu::Formattable::Formattable(double d)
    fmt3 = Formattable(0.0)
    assert fmt3.get_type() == Formattable.DOUBLE
    assert fmt3.get_double() == 0

    # [4]
    # icu::Formattable::Formattable(int32_t l)
    fmt4 = Formattable(INT32_MAX)
    assert fmt4.get_type() == Formattable.LONG
    assert fmt4.get_long() == INT32_MAX

    # [5]
    # icu::Formattable::Formattable(int64_t ll)
    fmt5 = Formattable(INT32_MAX + 1)
    assert fmt5.get_type() == Formattable.INT64
    assert fmt5.get_int64() == INT32_MAX + 1

    # [7]
    # icu::Formattable::Formattable(
    #       StringPiece number,
    #       UErrorCode &status
    # )
    fmt7 = Formattable("1e-3")
    assert fmt7.get_type() == Formattable.DOUBLE
    assert fmt7.get_double() == 1e-3

    # [8]
    # icu::Formattable::Formattable(const UnicodeString &strToCopy)
    fmt8 = Formattable(UnicodeString("1e-3"))
    assert fmt8.get_type() == Formattable.STRING
    assert fmt8.get_string() == "1e-3"

    # [10]
    # icu::Formattable::Formattable(
    #       const Formattable *arrayToCopy,
    #       int32_t count
    # )
    array = [fmt2, fmt3, fmt4, fmt5, fmt7, fmt8]
    fmt10 = Formattable(array, len(array))
    assert fmt10.get_type() == Formattable.ARRAY
    assert fmt10.get_array() == array

    fmt10a = Formattable([fmt10])
    assert fmt10a.get_type() == Formattable.ARRAY
    assert fmt10a.get_array() == [fmt10]
    assert fmt10a[0].get_type() == Formattable.ARRAY
    assert fmt10a[0].get_array() == array

    # [11]
    # icu::Formattable::Formattable(UObject *objectToAdopt)
    fmt11a = Formattable(Calendar.create_instance("en"))
    assert fmt11a.get_type() == Formattable.OBJECT
    assert isinstance(fmt11a.get_object(), Calendar)

    fmt11b = Formattable(CurrencyAmount(1, "JPY"))
    assert fmt11b.get_type() == Formattable.OBJECT
    assert isinstance(fmt11b.get_object(), CurrencyAmount)

    fmt11c = Formattable(DateInterval(0, 1))
    assert fmt11c.get_type() == Formattable.OBJECT
    assert isinstance(fmt11c.get_object(), DateInterval)

    fmt11d = Formattable(TimeUnitAmount(1, TimeUnit.UTIMEUNIT_DAY))
    assert fmt11d.get_type() == Formattable.OBJECT
    assert isinstance(fmt11d.get_object(), TimeUnitAmount)

    fmt11e = Formattable(TimeZone.create_time_zone("JST"))
    assert fmt11e.get_type() == Formattable.OBJECT
    assert isinstance(fmt11e.get_object(), TimeZone)

    # [12]
    # icu::Formattable::Formattable(const Formattable &)
    fmt12 = Formattable(fmt10)
    assert fmt12.get_type() == Formattable.ARRAY
    assert fmt12.get_array() == array


def test_get_object_upcasting():
    from icupy import BasicTimeZone, SimpleTimeZone

    fmt = Formattable(TimeZone.get_gmt())
    zone = fmt.get_object()
    assert isinstance(zone, SimpleTimeZone)

    # TimeZone -> BasicTimeZone
    fmt = Formattable(TimeZone.create_time_zone("JST"))
    zone = fmt.get_object()
    assert not isinstance(zone, SimpleTimeZone)
    assert isinstance(zone, BasicTimeZone)



import copy

from icupy import Locale, UnicodeString, StringEnumeration


def test_clone():
    # StringEnumeration *icu::Locale::createKeywords(UErrorCode &status)
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    it1 = loc.create_keywords()
    assert isinstance(it1, StringEnumeration)

    # int32_t icu::StringEnumeration::count(UErrorCode &status)
    assert it1.count() == len(it1) == 2

    # const char *icu::StringEnumeration::next(
    #       int32_t *resultLength,
    #       UErrorCode &status
    # )
    assert it1.next() is not None

    # StringEnumeration *icu::StringEnumeration::clone()
    it2 = it1.clone()
    assert isinstance(it2, StringEnumeration)
    assert len(it2) == 2
    assert it2.next() is not None
    assert it2.next() is None

    t2 = [x for x in iter(it2)]
    assert len(t2) == 2
    assert "calendar" in t2
    assert "collation" in t2

    it3 = copy.copy(it1)
    t3 = [x for x in iter(it3)]
    assert len(t3) == 2
    assert "calendar" in t3
    assert "collation" in t3

    it4 = copy.deepcopy(it1)
    t4 = [x for x in iter(it4)]
    assert len(t4) == 2
    assert "calendar" in t4
    assert "collation" in t4


def test_next():
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    it = loc.create_keywords()
    assert isinstance(it, StringEnumeration)
    assert it.count() == 2
    assert len(it) == 2

    # const char *icu::StringEnumeration::next(
    #       int32_t *resultLength,
    #       UErrorCode &status
    # )
    result = it.next()
    assert isinstance(result, str)
    assert result in ["calendar", "collation"]

    result = it.next()
    assert isinstance(result, str)
    assert result in ["calendar", "collation"]

    result = it.next()
    assert result is None

    assert "calendar" in it
    assert "collation" in it

    t = [x for x in iter(it)]
    assert len(t) == 2
    assert all(isinstance(x, str) for x in t)
    assert "calendar" in t
    assert "collation" in t


def test_operator():
    loc1 = Locale("de@calendar=buddhist;collation=phonebook")
    it1 = loc1.create_keywords()
    assert isinstance(it1, StringEnumeration)

    loc2 = loc1.clone()
    it2 = loc2.create_keywords()
    assert isinstance(it2, StringEnumeration)

    # UBool icu::StringEnumeration::operator!=(const StringEnumeration &that)
    assert not (it1 != it2)

    # UBool icu::StringEnumeration::operator==(const StringEnumeration &that)
    assert it1 == it2


def test_reset():
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    it = loc.create_keywords()
    assert isinstance(it, StringEnumeration)
    assert it.count() == 2
    assert len(it) == 2

    _ = it.next()
    _ = it.next()
    assert it.next() is None

    # void icu::StringEnumeration::reset(UErrorCode &status)
    it.reset()
    assert it.next() is not None


def test_snext():
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    it = loc.create_keywords()
    assert isinstance(it, StringEnumeration)
    assert it.count() == 2
    assert len(it) == 2

    # const UnicodeString *icu::StringEnumeration::snext(UErrorCode &status)
    result = it.snext()
    assert isinstance(result, UnicodeString)
    assert str(result) in ["calendar", "collation"]

    result = it.snext()
    assert isinstance(result, UnicodeString)
    assert str(result) in ["calendar", "collation"]

    result = it.snext()
    assert result is None

    assert UnicodeString("calendar", -1) in it
    assert UnicodeString("collation", -1) in it

    it.reset()
    t = [x.clone() for x in iter(it.snext, None)]
    assert len(t) == 2
    assert all(isinstance(x, UnicodeString) for x in t)
    t2 = [str(x) for x in t]
    assert "calendar" in t2
    assert "collation" in t2


def test_unext():
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    it = loc.create_keywords()
    assert isinstance(it, StringEnumeration)
    assert it.count() == 2
    assert len(it) == 2

    # const char16_t *icu::StringEnumeration::unext(
    #       int32_t *resultLength,
    #       UErrorCode &status
    # )
    result = it.unext()
    assert isinstance(result, str)
    assert result in ["calendar", "collation"]

    result = it.unext()
    assert isinstance(result, str)
    assert result in ["calendar", "collation"]

    result = it.unext()
    assert result is None

    assert "calendar" in it
    assert "collation" in it

    it.reset()
    t = [x for x in iter(it.unext, None)]
    assert len(t) == 2
    assert all(isinstance(x, str) for x in t)
    assert "calendar" in t
    assert "collation" in t



import pytest
from icupy import (
    Locale, U_ICU_VERSION_MAJOR_NUM,
    uenum_close, uenum_count, uenum_next, uenum_open_char_strings_enumeration,
    uenum_open_from_string_enumeration, uenum_open_uchar_strings_enumeration,
    uenum_reset, uenum_unext,
)

from . import gc


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 50, reason="ICU4C<50")
def test_open_char_strings_enumeration():
    strings = ["abc", "\uff11\uff12", "\U0001f338\U0001f339"]
    with gc(uenum_open_char_strings_enumeration(strings, len(strings)),
            uenum_close) as en:
        assert uenum_count(en) == 3
        assert uenum_next(en) == strings[0]
        assert uenum_next(en) == strings[1]
        assert uenum_next(en) == strings[2]
        assert uenum_next(en) is None
        uenum_reset(en)
        assert uenum_next(en) == strings[0]

    with gc(uenum_open_char_strings_enumeration(strings),
            uenum_close) as en:
        assert uenum_count(en) == 3
        assert uenum_next(en) == strings[0]
        assert uenum_next(en) == strings[1]
        assert uenum_next(en) == strings[2]
        assert uenum_next(en) is None
        uenum_reset(en)
        assert uenum_next(en) == strings[0]


def test_open_from_string_enumeration():
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    adopted = loc.create_keywords()
    with gc(uenum_open_from_string_enumeration(adopted), uenum_close) as en:
        assert uenum_count(en) == 2
        assert uenum_next(en) == "calendar"
        assert uenum_next(en) == "collation"
        assert uenum_next(en) is None
        uenum_reset(en)
        assert uenum_next(en) == "calendar"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 50, reason="ICU4C<50")
def test_open_uchar_strings_enumeration():
    strings = ["abc", "\uff11\uff12", "\U0001f338\U0001f339"]
    with gc(uenum_open_uchar_strings_enumeration(strings, len(strings)),
            uenum_close) as en:
        assert uenum_count(en) == 3
        assert uenum_unext(en) == strings[0]
        assert uenum_unext(en) == strings[1]
        assert uenum_unext(en) == strings[2]
        assert uenum_unext(en) is None
        uenum_reset(en)
        assert uenum_unext(en) == strings[0]

    with gc(uenum_open_uchar_strings_enumeration(strings),
            uenum_close) as en:
        assert uenum_count(en) == 3
        assert uenum_unext(en) == strings[0]
        assert uenum_unext(en) == strings[1]
        assert uenum_unext(en) == strings[2]
        assert uenum_unext(en) is None
        uenum_reset(en)
        assert uenum_unext(en) == strings[0]



import copy

import pytest
from icupy import (
    BreakIterator, Collator, ICUError, Locale, RuleBasedCollator,
    SearchIterator, StringCharacterIterator, StringSearch, UErrorCode,
    USEARCH_DONE, USearchAttribute, USearchAttributeValue, UnicodeString,
)


def test_api():
    pattern = UnicodeString("abab")
    text = UnicodeString("abababab")
    breakiter = BreakIterator.create_character_instance(Locale.get_us())
    coll = Collator.create_instance(Locale.get_us())
    it = StringSearch(pattern, text, coll, breakiter)

    assert USEARCH_DONE == StringSearch.DONE

    # int32_t icu::SearchIterator::first(UErrorCode &status)
    assert it.first() == 0

    # int32_t icu::SearchIterator::getMatchedLength(void)
    assert it.get_matched_length() == 4

    # int32_t icu::SearchIterator::getMatchedStart(void)
    assert it.get_matched_start() == 0

    # void icu::SearchIterator::getMatchedText(UnicodeString &result)
    result = UnicodeString()
    it.get_matched_text(result)
    assert result == "abab"

    # int32_t icu::StringSearch::getOffset(void)
    assert it.get_offset() == 0

    # int32_t icu::SearchIterator::following(
    #       int32_t position,
    #       UErrorCode &status
    # )
    assert it.following(0) == 0
    assert it.following(1) == 2
    assert it.following(3) == 4
    assert it.following(5) == USEARCH_DONE

    # int32_t icu::SearchIterator::last(UErrorCode &status)
    assert it.last() == 4

    # void icu::StringSearch::reset()
    it.reset()

    # int32_t icu::SearchIterator::next(UErrorCode &status)
    assert it.next() == 0
    assert it.next() == 4
    assert it.next() == USEARCH_DONE

    assert list(it) == [0, 4]

    # int32_t icu::SearchIterator::preceding(
    #       int32_t position,
    #       UErrorCode &status
    # )
    assert it.preceding(8) == 4
    assert it.preceding(7) == 2
    assert it.preceding(5) == 0
    assert it.preceding(3) == USEARCH_DONE

    # void icu::StringSearch::setOffset(
    #       int32_t position,
    #       UErrorCode &status
    # )
    it.set_offset(8)

    # int32_t icu::SearchIterator::previous(UErrorCode &status)
    assert it.previous() == 4
    assert it.previous() == 0
    assert it.previous() == USEARCH_DONE

    assert reversed(it) == [4, 0]

    assert it.get_offset() == 0
    it.set_offset(4)
    assert it.get_offset() == 4
    with pytest.raises(ICUError) as exc_info:
        it.set_offset(9)
    assert exc_info.value.args[0] == UErrorCode.U_INDEX_OUTOFBOUNDS_ERROR

    # USearchAttributeValue icu::SearchIterator::getAttribute(
    #       USearchAttribute attribute
    # )
    assert (it.get_attribute(USearchAttribute.USEARCH_OVERLAP)
            == USearchAttributeValue.USEARCH_OFF)

    # void icu::SearchIterator::setAttribute(
    #       USearchAttribute attribute,
    #       USearchAttributeValue value,
    #       UErrorCode &status
    # )
    it.set_attribute(USearchAttribute.USEARCH_OVERLAP,
                     USearchAttributeValue.USEARCH_ON)
    assert (it.get_attribute(USearchAttribute.USEARCH_OVERLAP)
            == USearchAttributeValue.USEARCH_ON)

    # const BreakIterator *icu::SearchIterator::getBreakIterator(void)
    result = it.get_break_iterator()
    assert isinstance(result, BreakIterator)
    assert result == breakiter

    # void icu::SearchIterator::setBreakIterator(
    #       BreakIterator *breakiter,
    #       UErrorCode &status
    # )
    breakiter2 = BreakIterator.create_character_instance(Locale.get_japan())
    assert breakiter2 != breakiter
    it.set_break_iterator(breakiter2)
    assert it.get_break_iterator() == breakiter2

    # RuleBasedCollator *icu::StringSearch::getCollator()
    result = it.get_collator()
    assert isinstance(result, RuleBasedCollator)
    assert result == coll

    # void icu::StringSearch::setCollator(
    #       RuleBasedCollator *coll,
    #       UErrorCode &status
    # )
    coll2 = Collator.create_instance(Locale.get_japan())
    assert coll2 != coll
    it.set_collator(coll2)
    assert it.get_collator() == coll2

    # const UnicodeString &icu::StringSearch::getPattern()
    result = it.get_pattern()
    assert isinstance(result, UnicodeString)
    assert result == pattern

    # void icu::StringSearch::setPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    pattern2 = UnicodeString("ab")
    it.set_pattern(pattern2)
    assert it.get_pattern() == pattern2

    it.set_pattern("abc")
    assert it.get_pattern() == "abc"

    # const UnicodeString &icu::SearchIterator::getText(void)
    result = it.get_text()
    assert isinstance(result, UnicodeString)
    assert result == text


def test_clone():
    pattern = UnicodeString("fox")
    text = UnicodeString("The quick brown fox jumps over the lazy dog.")
    breakiter = BreakIterator.create_word_instance(Locale.get_us())
    coll = Collator.create_instance(Locale.get_us())
    test1 = StringSearch(pattern, text, coll, breakiter)
    assert test1.next() != USEARCH_DONE

    # StringSearch *icu::StringSearch::clone()
    test2 = test1.clone()
    assert isinstance(test2, StringSearch)
    assert test2 != test1
    assert test2.next() != USEARCH_DONE
    assert test2 == test1

    test3 = copy.copy(test1)
    assert test3 != test1
    assert test3.next() != USEARCH_DONE
    assert test3 == test1

    test4 = copy.deepcopy(test1)
    assert test4 != test1
    assert test4.next() != USEARCH_DONE
    assert test4 == test1

    # StringSearch *icu::StringSearch::safeClone()
    test5 = test1.safe_clone()
    assert isinstance(test5, StringSearch)
    assert test5 == test1


def test_operator():
    pattern = UnicodeString("fox")
    text = UnicodeString("The quick brown fox jumps over the lazy dog.")
    breakiter = BreakIterator.create_word_instance(Locale.get_us())
    coll = Collator.create_instance(Locale.get_us())
    test1 = StringSearch(pattern, text, coll, breakiter)
    test2 = StringSearch(pattern, text, coll, breakiter)

    # UBool icu::SearchIterator::operator!=(const SearchIterator &that)
    assert not (test1 != test2)

    # UBool icu::StringSearch::operator==(const SearchIterator &that)
    assert test1 == test2

    assert test1.next() != USEARCH_DONE
    assert test1 != test2
    assert not (test1 == test2)

    assert test2.next() != USEARCH_DONE
    assert not (test1 != test2)
    assert test1 == test2


def test_set_text():
    pattern = UnicodeString("a")
    text = UnicodeString("abc")
    coll = Collator.create_instance(Locale.get_us())
    it = StringSearch(pattern, text, coll, None)

    # [1]
    # void icu::StringSearch::setText(
    #       CharacterIterator &text,
    #       UErrorCode &status
    # )
    s = UnicodeString("ab c")
    text = StringCharacterIterator(s)
    it.set_text(text)
    assert it.get_text() == s

    # [2]
    # void icu::StringSearch::setText(
    #       const UnicodeString &text,
    #       UErrorCode &status
    # )
    text = UnicodeString("a bc")
    it.set_text(text)
    assert it.get_text() == text

    text = "a\U0001f338b"
    it.set_text(text)
    assert it.get_text() == text


def test_string_search():
    assert issubclass(StringSearch, SearchIterator)

    pattern = UnicodeString("fox")
    text1 = UnicodeString("The quick brown fox jumps over the lazy dog.")
    breakiter = BreakIterator.create_word_instance(Locale.get_us())
    coll = Collator.create_instance(Locale.get_us())
    text2 = StringCharacterIterator(text1)

    # [1]
    # icu::StringSearch::StringSearch(
    #       const UnicodeString &pattern,
    #       const UnicodeString &text,
    #       const Locale &locale,
    #       BreakIterator *breakiter,
    #       UErrorCode &status
    # )
    test1 = StringSearch(pattern, text1, Locale.get_us(), None)
    assert test1.get_break_iterator() is None
    assert test1.get_collator()
    assert test1.get_pattern() == pattern
    assert test1.get_text() == text1

    test1b = StringSearch(pattern, text1, Locale.get_us(), breakiter)
    assert test1b.get_break_iterator() == breakiter
    assert test1b.get_collator()
    assert test1b.get_pattern() == pattern
    assert test1b.get_text() == text1

    test1c = StringSearch(str(pattern), text1, Locale.get_us(), breakiter)
    assert test1c.get_break_iterator() == breakiter
    assert test1c.get_collator()
    assert test1c.get_pattern() == pattern
    assert test1c.get_text() == text1

    test1d = StringSearch(pattern, text1, "en_US", breakiter)
    assert test1d.get_break_iterator() == breakiter
    assert test1d.get_collator()
    assert test1d.get_pattern() == pattern
    assert test1d.get_text() == text1

    test1e = StringSearch(str(pattern), text1, "en_US", breakiter)
    assert test1e.get_break_iterator() == breakiter
    assert test1e.get_collator()
    assert test1e.get_pattern() == pattern
    assert test1e.get_text() == text1

    test1f = StringSearch(pattern, str(text1), Locale.get_us(), breakiter)
    assert test1f.get_break_iterator() == breakiter
    assert test1f.get_collator()
    assert test1f.get_pattern() == pattern
    assert test1f.get_text() == text1

    test1g = StringSearch(str(pattern), str(text1), Locale.get_us(), breakiter)
    assert test1g.get_break_iterator() == breakiter
    assert test1g.get_collator()
    assert test1g.get_pattern() == pattern
    assert test1g.get_text() == text1

    test1h = StringSearch(pattern, str(text1), "en_US", breakiter)
    assert test1h.get_break_iterator() == breakiter
    assert test1h.get_collator()
    assert test1h.get_pattern() == pattern
    assert test1h.get_text() == text1

    test1i = StringSearch(str(pattern), str(text1), "en_US", breakiter)
    assert test1i.get_break_iterator() == breakiter
    assert test1i.get_collator()
    assert test1i.get_pattern() == pattern
    assert test1i.get_text() == text1

    # [2]
    # icu::StringSearch::StringSearch(
    #       const UnicodeString &pattern,
    #       const UnicodeString &text,
    #       RuleBasedCollator *coll,
    #       BreakIterator *breakiter,
    #       UErrorCode &status
    # )
    test2 = StringSearch(pattern, text1, coll, None)
    assert test2.get_break_iterator() is None
    assert test2.get_collator() == coll
    assert test2.get_pattern() == pattern
    assert test2.get_text() == text1

    test2b = StringSearch(pattern, text1, coll, breakiter)
    assert test2b.get_break_iterator() == breakiter
    assert test2b.get_collator() == coll
    assert test2b.get_pattern() == pattern
    assert test2b.get_text() == text1

    test2c = StringSearch(str(pattern), text1, coll, breakiter)
    assert test2c.get_break_iterator() == breakiter
    assert test2c.get_collator() == coll
    assert test2c.get_pattern() == pattern
    assert test2c.get_text() == text1

    test2d = StringSearch(pattern, str(text1), coll, breakiter)
    assert test2d.get_break_iterator() == breakiter
    assert test2d.get_collator() == coll
    assert test2d.get_pattern() == pattern
    assert test2d.get_text() == text1

    test2e = StringSearch(str(pattern), str(text1), coll, breakiter)
    assert test2e.get_break_iterator() == breakiter
    assert test2e.get_collator() == coll
    assert test2e.get_pattern() == pattern
    assert test2e.get_text() == text1

    # [3]
    # icu::StringSearch::StringSearch(
    #       const UnicodeString &pattern,
    #       CharacterIterator &text,
    #       const Locale &locale,
    #       BreakIterator *breakiter,
    #       UErrorCode &status
    # )
    test3 = StringSearch(pattern, text2, Locale.get_us(), None)
    assert test3.get_break_iterator() is None
    assert test3.get_collator()
    assert test3.get_pattern() == pattern
    assert test3.get_text() == text1

    test3b = StringSearch(pattern, text2, Locale.get_us(), breakiter)
    assert test3b.get_break_iterator() == breakiter
    assert test3b.get_collator()
    assert test3b.get_pattern() == pattern
    assert test3b.get_text() == text1

    test3c = StringSearch(str(pattern), text2, Locale.get_us(), breakiter)
    assert test3c.get_break_iterator() == breakiter
    assert test3c.get_collator()
    assert test3c.get_pattern() == pattern
    assert test3c.get_text() == text1

    test3d = StringSearch(pattern, text2, "en_US", breakiter)
    assert test3d.get_break_iterator() == breakiter
    assert test3d.get_collator()
    assert test3d.get_pattern() == pattern
    assert test3d.get_text() == text1

    test3e = StringSearch(str(pattern), text2, "en_US", breakiter)
    assert test3e.get_break_iterator() == breakiter
    assert test3e.get_collator()
    assert test3e.get_pattern() == pattern
    assert test3e.get_text() == text1

    # [4]
    # icu::StringSearch::StringSearch(
    #       const UnicodeString &pattern,
    #       CharacterIterator &text,
    #       RuleBasedCollator *coll,
    #       BreakIterator *breakiter,
    #       UErrorCode &status
    # )
    test4 = StringSearch(pattern, text2, coll, None)
    assert test4.get_break_iterator() is None
    assert test4.get_collator() == coll
    assert test4.get_pattern() == pattern
    assert test4.get_text() == text1

    test4b = StringSearch(pattern, text2, coll, breakiter)
    assert test4b.get_break_iterator() == breakiter
    assert test4b.get_collator() == coll
    assert test4b.get_pattern() == pattern
    assert test4b.get_text() == text1

    test4c = StringSearch(str(pattern), text2, coll, breakiter)
    assert test4c.get_break_iterator() == breakiter
    assert test4c.get_collator() == coll
    assert test4c.get_pattern() == pattern
    assert test4c.get_text() == text1

    # [5]
    # icu::StringSearch::StringSearch(const StringSearch &that)
    test5 = StringSearch(test2b)
    assert test5.get_break_iterator() == breakiter
    assert test5.get_collator() == coll
    assert test5.get_pattern() == pattern
    assert test5.get_text() == text1



import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 62:
    pytest.skip("ICU4C<62", allow_module_level=True)

import copy

from icupy import CurrencyUnit, Locale, MeasureUnit
from icupy.number import NumberFormatter


def test_api():
    assert issubclass(CurrencyUnit, MeasureUnit)

    # [2]
    # icu::CurrencyUnit::CurrencyUnit(
    #       ConstChar16Ptr isoCode,
    #       UErrorCode &ec
    # )
    unit1 = CurrencyUnit("USD")
    unit2 = CurrencyUnit("JPY")

    # [4]
    # icu::CurrencyUnit::CurrencyUnit(const CurrencyUnit &other)
    unit3 = CurrencyUnit(unit1)

    # UBool icu::MeasureUnit::operator!=(const UObject &other)
    assert unit1 != unit2
    assert not (unit1 != unit3)
    assert unit2 != unit3

    # UBool icu::MeasureUnit::operator==(const UObject &other)
    assert not (unit1 == unit2)
    assert unit1 == unit3
    assert not (unit2 == unit3)

    # const char16_t *icu::CurrencyUnit::getISOCurrency()
    iso_code = unit3.get_iso_currency()
    assert isinstance(iso_code, str)
    assert iso_code == "USD"

    iso_code = unit2.get_iso_currency()
    assert isinstance(iso_code, str)
    assert iso_code == "JPY"

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unit(
    #       const MeasureUnit &unit
    # )
    fmt = NumberFormatter.with_locale(Locale.get_us())
    assert fmt.unit(unit1).format_double(100).to_string() == "$100.00"
    assert fmt.unit(unit2).format_double(100).to_string() == "\xa5100"  # ¥100


def test_clone():
    unit1 = CurrencyUnit("USD")

    # CurrencyUnit *icu::CurrencyUnit::clone()
    unit2 = unit1.clone()
    assert isinstance(unit2, CurrencyUnit)
    assert unit2 == unit1

    unit3 = copy.copy(unit1)
    assert unit3 == unit1

    unit4 = copy.deepcopy(unit1)
    assert unit4 == unit1



import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 62:
    pytest.skip("ICU4C<62", allow_module_level=True)

import copy

from icupy import Locale, MeasureUnit, TimeUnit
from icupy.number import NumberFormatter


def test_api():
    assert issubclass(TimeUnit, MeasureUnit)

    # static TimeUnit *icu::TimeUnit::createInstance(
    #       UTimeUnitFields timeUnitField,
    #       UErrorCode &status
    # )
    unit1 = TimeUnit.create_instance(TimeUnit.UTIMEUNIT_MONTH)
    assert isinstance(unit1, TimeUnit)

    unit2 = TimeUnit.create_instance(TimeUnit.UTIMEUNIT_WEEK)
    assert isinstance(unit2, TimeUnit)

    # icu::TimeUnit::TimeUnit(const TimeUnit &other)
    unit3 = TimeUnit(unit1)

    # UBool icu::MeasureUnit::operator!=(const UObject &other)
    assert unit1 != unit2
    assert not (unit1 != unit3)
    assert unit2 != unit3

    # UBool icu::MeasureUnit::operator==(const UObject &other)
    assert not (unit1 == unit2)
    assert unit1 == unit3
    assert not (unit2 == unit3)

    # FIXME: MeasureUnit.__eq__(self, other: UObject) is not work.
    # unit4 = MeasureUnit.create_month()
    # assert unit1 == unit4

    # UTimeUnitFields icu::TimeUnit::getTimeUnitField()
    assert unit3.get_time_unit_field() == TimeUnit.UTIMEUNIT_MONTH
    assert unit2.get_time_unit_field() == TimeUnit.UTIMEUNIT_WEEK

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unit(
    #       const MeasureUnit &unit
    # )
    fmt = NumberFormatter.with_locale(Locale.get_us())
    assert fmt.unit(unit1).format_int(1).to_string() == "1 mth"
    assert fmt.unit(unit2).format_int(1).to_string() == "1 wk"


def test_clone():
    unit1 = TimeUnit.create_instance(TimeUnit.UTIMEUNIT_MONTH)

    # TimeUnit *icu::TimeUnit::clone()
    unit2 = unit1.clone()
    assert isinstance(unit2, TimeUnit)
    assert unit2 == unit1

    unit3 = copy.copy(unit1)
    assert unit3 == unit1

    unit4 = copy.deepcopy(unit1)
    assert unit4 == unit1



import pytest
from icupy import (
    DateFormatSymbols, Locale, ULocDataLocaleType, U_ICU_VERSION_MAJOR_NUM,
    UnicodeString,
)


def test_api():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))

    # Locale icu::DateFormatSymbols::getLocale(
    #       ULocDataLocaleType type,
    #       UErrorCode &status
    # )
    locale = sym.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
    assert isinstance(locale, Locale)
    assert locale == Locale("ja_JP")

    # UnicodeString &icu::DateFormatSymbols::getLocalPatternChars(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    chars = sym.get_local_pattern_chars(result)
    assert isinstance(chars, UnicodeString)
    assert id(chars) == id(result)
    assert len(chars) > 0

    # void icu::DateFormatSymbols::setLocalPatternChars(
    #       const UnicodeString &newLocalPatternChars
    # )
    sym2.set_local_pattern_chars(UnicodeString("abcdefghijklmonpqr"))
    chars = sym2.get_local_pattern_chars(result)
    assert chars == "abcdefghijklmonpqr"

    sym2.set_local_pattern_chars("GyMdkHmsSEDFwWahKzYeugAZvcLQqVUOXxrbB")
    chars = sym2.get_local_pattern_chars(result)
    assert chars == "GyMdkHmsSEDFwWahKzYeugAZvcLQqVUOXxrbB"

    # static const char16_t *icu::DateFormatSymbols::getPatternUChars(void)
    uchars = DateFormatSymbols.get_pattern_uchars()
    assert isinstance(uchars, str)
    assert len(uchars) > 0


def test_date_format_symbols():
    # [1]
    # icu::DateFormatSymbols::DateFormatSymbols(UErrorCode &status)
    sym1 = DateFormatSymbols()

    # [2]
    # icu::DateFormatSymbols::DateFormatSymbols(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    sym2 = DateFormatSymbols(Locale.get_us())

    # [5]
    # icu::DateFormatSymbols::DateFormatSymbols(const DateFormatSymbols &)
    sym5 = DateFormatSymbols(sym2)

    # icu::DateFormatSymbols::operator!=(const DateFormatSymbols &other)
    assert sym1 != sym2
    assert sym1 != sym5
    assert not (sym2 != sym5)

    # icu::DateFormatSymbols::operator==(const DateFormatSymbols &other)
    assert not (sym1 == sym2)
    assert not (sym1 == sym5)
    assert sym2 == sym5


def test_get_am_pm_strings():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getAmPmStrings(
    #       int32_t &count
    # )
    ampms = sym.get_am_pm_strings()
    assert isinstance(ampms, list)
    assert len(ampms) == 2
    assert all(isinstance(x, UnicodeString) for x in ampms)
    assert ampms[0] == "\u5348\u524d"  # 午前
    assert ampms[1] == "\u5348\u5f8c"  # 午後

    # void icu::DateFormatSymbols::setAmPmStrings(
    #       const UnicodeString *ampms,
    #       int32_t count
    # )
    assert sym2.get_am_pm_strings() != ampms
    sym2.set_am_pm_strings(ampms, len(ampms))
    assert sym2.get_am_pm_strings() == ampms

    assert sym3.get_am_pm_strings() != ampms
    sym3.set_am_pm_strings(ampms)  # count is optional
    assert sym3.get_am_pm_strings() == ampms


def test_get_era_names():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getEraNames(int32_t &count)
    era_names = sym.get_era_names()
    assert isinstance(era_names, list)
    assert len(era_names) == 2
    assert all(isinstance(x, UnicodeString) for x in era_names)
    assert era_names[0] == "\u7D00\u5143\u524D"  # 紀元前
    assert era_names[1] == "\u897F\u66A6"  # 西暦

    # void icu::DateFormatSymbols::setEraNames(
    #       const UnicodeString *eraNames,
    #       int32_t count
    # )
    assert sym2.get_era_names() != era_names
    sym2.set_era_names(era_names, len(era_names))
    assert sym2.get_era_names() == era_names

    assert sym3.get_era_names() != era_names
    sym3.set_era_names(era_names)  # count is optional
    assert sym3.get_era_names() == era_names


def test_get_eras():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getEras(int32_t &count)
    eras = sym.get_eras()
    assert isinstance(eras, list)
    assert len(eras) == 2
    assert all(isinstance(x, UnicodeString) for x in eras)
    assert eras[0] == "\u7D00\u5143\u524D"  # 紀元前
    assert eras[1] == "\u897F\u66A6"  # 西暦

    # void icu::DateFormatSymbols::setEras(
    #       const UnicodeString *eras,
    #       int32_t count
    # )
    assert sym2.get_eras() != eras
    sym2.set_eras(eras, len(eras))
    assert sym2.get_eras() == eras

    assert sym3.get_eras() != eras
    sym3.set_eras(eras)  # count is optional
    assert sym3.get_eras() == eras


def test_get_months():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # [1]
    # const UnicodeString *icu::DateFormatSymbols::getMonths(int32_t &count)
    months1 = sym.get_months()
    assert isinstance(months1, list)
    assert len(months1) == 12
    assert all(isinstance(x, UnicodeString) for x in months1)
    assert months1[0] == "1\u6708"  # 1月
    assert months1[11] == "12\u6708"  # 12月

    # [1]
    # void icu::DateFormatSymbols::setMonths(
    #       const UnicodeString *months,
    #       int32_t count
    # )
    assert sym2.get_months() != months1
    sym2.set_months(months1, len(months1))
    assert sym2.get_months() == months1

    assert sym3.get_months() != months1
    sym3.set_months(months1)  # count is optional
    assert sym3.get_months() == months1

    # [2]
    # const UnicodeString *icu::DateFormatSymbols::getMonths(
    #       int32_t &count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    months2 = sym.get_months(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert isinstance(months2, list)
    assert len(months2) == 12
    assert all(isinstance(x, UnicodeString) for x in months2)
    assert months2[0] == "1\u6708"  # 1月
    assert months2[11] == "12\u6708"  # 12月

    # [2]
    # void icu::DateFormatSymbols::setMonths(
    #       const UnicodeString *months,
    #       int32_t count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    assert sym2.get_months(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != months2
    sym2.set_months(
        months2,
        len(months2),
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym2.get_months(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == months2

    assert sym3.get_months(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != months2
    sym3.set_months(
        months2,
        -1,
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym3.get_months(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == months2


def test_get_narrow_eras():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getNarrowEras(
    #       int32_t &count
    # )
    narrow_eras = sym.get_narrow_eras()
    assert isinstance(narrow_eras, list)
    assert len(narrow_eras) == 2
    assert all(isinstance(x, UnicodeString) for x in narrow_eras)
    assert narrow_eras[0] == "BC"
    assert narrow_eras[1] == "AD"

    # void icu::DateFormatSymbols::setNarrowEras(
    #       const UnicodeString *narrowEras,
    #       int32_t count
    # )
    assert sym2.get_narrow_eras() != narrow_eras
    sym2.set_narrow_eras(narrow_eras, len(narrow_eras))
    assert sym2.get_narrow_eras() == narrow_eras

    assert sym3.get_narrow_eras() != narrow_eras
    sym3.set_narrow_eras(narrow_eras)  # count is optional
    assert sym3.get_narrow_eras() == narrow_eras


def test_get_quarters():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getQuarters(
    #       int32_t &count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    quarters = sym.get_quarters(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE)
    assert isinstance(quarters, list)
    assert len(quarters) == 4
    assert all(isinstance(x, UnicodeString) for x in quarters)
    assert quarters[0] == "\u7B2C1\u56DB\u534A\u671F"  # 第1四半期
    assert quarters[3] == "\u7B2C4\u56DB\u534A\u671F"  # 第4四半期

    # void icu::DateFormatSymbols::setQuarters(
    #       const UnicodeString *quarters,
    #       int32_t count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    assert sym2.get_quarters(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE) != quarters
    sym2.set_quarters(
        quarters,
        len(quarters),
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE)
    assert sym2.get_quarters(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE) == quarters

    assert sym3.get_quarters(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE) != quarters
    sym3.set_quarters(
        quarters,
        -1,
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE)
    assert sym3.get_quarters(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.WIDE) == quarters


def test_get_short_months():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getShortMonths(
    #       int32_t &count
    # )
    short_months = sym.get_short_months()
    assert isinstance(short_months, list)
    assert len(short_months) == 12
    assert all(isinstance(x, UnicodeString) for x in short_months)
    assert short_months[0] == "1\u6708"  # 1月
    assert short_months[11] == "12\u6708"  # 12月

    # void icu::DateFormatSymbols::setShortMonths(
    #       const UnicodeString *shortMonths,
    #       int32_t count
    # )
    assert sym2.get_short_months() != short_months
    sym2.set_short_months(short_months, len(short_months))
    assert sym2.get_short_months() == short_months

    assert sym3.get_short_months() != short_months
    sym3.set_short_months(short_months)  # count is optional
    assert sym3.get_short_months() == short_months


def test_get_short_weekdays():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getShortWeekdays(
    #       int32_t &count
    # )
    abbrev_weekdays = sym.get_short_weekdays()
    assert isinstance(abbrev_weekdays, list)
    assert len(abbrev_weekdays) == 8
    assert all(isinstance(x, UnicodeString) for x in abbrev_weekdays)
    assert len(abbrev_weekdays[0]) == 0
    assert abbrev_weekdays[1] == "\u65E5"  # 日
    assert abbrev_weekdays[7] == "\u571F"  # 土

    # void icu::DateFormatSymbols::setShortWeekdays(
    #       const UnicodeString *abbrevWeekdays,
    #       int32_t count
    # )
    assert sym2.get_short_weekdays() != abbrev_weekdays
    sym2.set_short_weekdays(abbrev_weekdays, len(abbrev_weekdays))
    assert sym2.get_short_weekdays() == abbrev_weekdays

    assert sym3.get_short_weekdays() != abbrev_weekdays
    sym3.set_short_weekdays(abbrev_weekdays)  # count is optional
    assert sym3.get_short_weekdays() == abbrev_weekdays


def test_get_weekdays():
    sym = DateFormatSymbols(Locale.get_japan())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # [1]
    # const UnicodeString *icu::DateFormatSymbols::getWeekdays(int32_t &count)
    weekdays1 = sym.get_weekdays()
    assert isinstance(weekdays1, list)
    assert len(weekdays1) == 8
    assert all(isinstance(x, UnicodeString) for x in weekdays1)
    assert len(weekdays1[0]) == 0
    assert weekdays1[1] == "\u65E5\u66DC\u65E5"  # 日曜日
    assert weekdays1[7] == "\u571F\u66DC\u65E5"  # 土曜日

    # [1]
    # void icu::DateFormatSymbols::setWeekdays(
    #       const UnicodeString *weekdays,
    #       int32_t count
    # )
    assert sym2.get_weekdays() != weekdays1
    sym2.set_weekdays(weekdays1, len(weekdays1))
    assert sym2.get_weekdays() == weekdays1

    assert sym3.get_weekdays() != weekdays1
    sym3.set_weekdays(weekdays1)  # count is optional
    assert sym3.get_weekdays() == weekdays1

    # [2]
    # const UnicodeString *icu::DateFormatSymbols::getWeekdays(
    #       int32_t &count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    weekdays2 = sym.get_weekdays(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert isinstance(weekdays2, list)
    assert len(weekdays2) == 8
    assert all(isinstance(x, UnicodeString) for x in weekdays2)
    assert len(weekdays2[0]) == 0
    assert weekdays2[1] == "\u65E5"  # 日
    assert weekdays2[7] == "\u571F"  # 土

    # [2]
    # void icu::DateFormatSymbols::setWeekdays(
    #       const UnicodeString *weekdays,
    #       int32_t count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    assert sym2.get_weekdays(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != weekdays2
    sym2.set_weekdays(
        weekdays2,
        len(weekdays2),
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym2.get_weekdays(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == weekdays2

    assert sym3.get_weekdays(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != weekdays2
    sym3.set_weekdays(
        weekdays2,
        -1,
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym3.get_weekdays(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == weekdays2


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_get_year_names():
    sym = DateFormatSymbols(Locale.get_china())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getYearNames(
    #       int32_t &count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    year_names = sym.get_year_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert isinstance(year_names, list)
    assert len(year_names) == 0

    # https://en.wikipedia.org/wiki/Sexagenary_cycle#Sexagenary_years
    src = [
        # "甲子", "乙丑", "丙寅",
        "\u7532\u5b50", "\u4e59\u4e11", "\u4e19\u5bc5",
        # "丁卯", "戊辰", "己巳",
        "\u4e01\u536f", "\u620a\u8fb0", "\u5df1\u5df3",
        # "庚午", "辛未", "壬申",
        "\u5e9a\u5348", "\u8f9b\u672a", "\u58ec\u7533",
        # "癸酉", "甲戌", "乙亥",
        "\u7678\u9149", "\u7532\u620c", "\u4e59\u4ea5",
        # "丙子", "丁丑", "戊寅",
        "\u4e19\u5b50", "\u4e01\u4e11", "\u620a\u5bc5",
        # "己卯", "庚辰", "辛巳",
        "\u5df1\u536f", "\u5e9a\u8fb0", "\u8f9b\u5df3",
        # "壬午", "癸未", "甲申",
        "\u58ec\u5348", "\u7678\u672a", "\u7532\u7533",
        # "乙酉", "丙戌", "丁亥",
        "\u4e59\u9149", "\u4e19\u620c", "\u4e01\u4ea5",
        # "戊子", "己丑", "庚寅",
        "\u620a\u5b50", "\u5df1\u4e11", "\u5e9a\u5bc5",
        # "辛卯", "壬辰", "癸巳",
        "\u8f9b\u536f", "\u58ec\u8fb0", "\u7678\u5df3",
        # "甲午", "乙未", "丙申",
        "\u7532\u5348", "\u4e59\u672a", "\u4e19\u7533",
        # "丁酉", "戊戌", "己亥",
        "\u4e01\u9149", "\u620a\u620c", "\u5df1\u4ea5",
        # "庚子", "辛丑", "壬寅",
        "\u5e9a\u5b50", "\u8f9b\u4e11", "\u58ec\u5bc5",
        # "癸卯", "甲辰", "乙巳",
        "\u7678\u536f", "\u7532\u8fb0", "\u4e59\u5df3",
        # "丙午", "丁未", "戊申",
        "\u4e19\u5348", "\u4e01\u672a", "\u620a\u7533",
        # "己酉", "庚戌", "辛亥",
        "\u5df1\u9149", "\u5e9a\u620c", "\u8f9b\u4ea5",
        # "壬子", "癸丑", "甲寅",
        "\u58ec\u5b50", "\u7678\u4e11", "\u7532\u5bc5",
        # "乙卯", "丙辰", "丁巳",
        "\u4e59\u536f", "\u4e19\u8fb0", "\u4e01\u5df3",
        # "戊午", "己未", "庚申",
        "\u620a\u5348", "\u5df1\u672a", "\u5e9a\u7533",
        # "辛酉", "壬戌", "癸亥",
        "\u8f9b\u9149", "\u58ec\u620c", "\u7678\u4ea5",
    ]
    year_names = [UnicodeString(s) for s in src]

    # void icu::DateFormatSymbols::setYearNames(
    #       const UnicodeString *yearNames,
    #       int32_t count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    assert sym2.get_year_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != year_names
    sym2.set_year_names(
        year_names,
        len(year_names),
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym2.get_year_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == year_names

    assert sym3.get_year_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != year_names
    sym3.set_year_names(
        year_names,
        -1,
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym3.get_year_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == year_names


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_get_zodiac_names():
    sym = DateFormatSymbols(Locale.get_china())
    sym2 = DateFormatSymbols(Locale("und"))
    sym3 = DateFormatSymbols(Locale("und"))

    # const UnicodeString *icu::DateFormatSymbols::getZodiacNames(
    #       int32_t &count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    zodiac_names = sym.get_zodiac_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert isinstance(zodiac_names, list)
    assert len(zodiac_names) == 0

    # https://en.wikipedia.org/wiki/Chinese_zodiac
    src = [
        "Rat", "Ox", "Tiger", "Rabbit", "Dragon", "Snake",
        "Horse", "Goat", "Monkey", "Rooster", "Dog", "Pig",
    ]
    zodiac_names = [UnicodeString(s) for s in src]

    # void icu::DateFormatSymbols::setZodiacNames(
    #       const UnicodeString *zodiacNames,
    #       int32_t count,
    #       DtContextType context,
    #       DtWidthType width
    # )
    assert sym2.get_zodiac_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != zodiac_names
    sym2.set_zodiac_names(
        zodiac_names,
        len(zodiac_names),
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym2.get_zodiac_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == zodiac_names

    assert sym3.get_zodiac_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) != zodiac_names
    sym3.set_zodiac_names(
        zodiac_names,
        -1,
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED)
    assert sym3.get_zodiac_names(
        DateFormatSymbols.FORMAT,
        DateFormatSymbols.ABBREVIATED) == zodiac_names
